#$Id$
#$Copyright$

package VRTS::Enclosure::Discover::HiCommand;
use strict;
use Carp;
require Exporter;
require XML::Simple;
require VRTS::Paths;
require VRTS::Enclosure::Utils;

my $ch;
my $ch1;
my $vardir;
my $user_agent = 'Hitachi:HiCommandCLI:0';
my ($probe_timeout, $protocol);

my %capabilities = ("VOL_CAPABLE" => 0, "SHOW_RG_LDEV" => 1, "SHOW_RG_PDEV" => 1, "SHOW_PDEV" => 1, "SHOW_LDEV" => 1,
   "SHOW_REPLICATION" => 1, "SHOW_TP_SOURCELDEV" => 1, "SHOW_TP_THINLDEV" => 1, "SHOW_LDEV_PDEV" => 0, "LDEV_PDEV_MAP" => 0,
   "PRIMARY_REMOTE_REP_CAPABLE" => 1, "SHOW_VSET" => 1, "SHOW_TP" => 1, "SHOW_RG" => 1, "SECONDARY_REMOTE_REP_CAPABLE" => 1,
   "SHOW_PDEV_FOR_THIN_LDEV" => 0, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 0, "SHOW_DEVICE_GROUP_REPLICATION" => 0);


sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::Discover::HiCommand;
    $vardir= VRTS::Paths::get_path("AppDir");
    return $module;

}

sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;

    $ch = $params->{"control-host"};
    $ch1 = $params->{"secondary-ch"};
    my $id = $params->{ID};
    my $server = $params->{"server"};
    my $port = $params->{"port"};
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    $protocol = $params->{'ARRAY_HITACHI_PROTOCOL'};
    $probe_timeout = $self->{"probe_timeout"};

    VRTS::Logger::log('trace', "control-host = [$ch]"); 
    VRTS::Logger::log('trace', "ID = [$id]"); 
    VRTS::Logger::log('trace', "server = [$server]"); 
    VRTS::Logger::log('trace', "port = [$port]"); 
    VRTS::Logger::log('trace', "user = [$user]"); 

    my $api_version;
    #### Get Enclosure Capabilities
    my $out1 = _get_array_capabilities($ch,$id,$server,$port,$user,$pass);
    if ($out1 =~ /^\d+$/)
    {
        my $err = $out1 >> 8;
        if ($err == 192)
        {
            VRTS::Logger::log('error', 'Request timed out due to no activity for timeout period');
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
    }
    if (($out1 eq '') || ($out1 =~ /Server is not/) || ($out1 =~ /Could not connect to server/))
    {
        if (defined ($ch1) && $ch1 ne "")
        {
            VRTS::Logger::log('error', 'Unable to contact HiCommand from primary, trying from secondary');
            # Contact HiCommand through secondary control host
            $out1 = _get_array_capabilities($ch1,$id,$server,$port,$user,$pass);
            if ($out1 eq "" || $out1 =~ /Server is not/)
            {
                 VRTS::Logger::log('error', 'Unable to contact HiCommand Server from primary as well as secondary');
                 return undef,&VRTS::Enclosure::Discover::CONNECT_FAILED;
            }
        }
        else
        {
            VRTS::Logger::log('error', 'Unable to contact HiCommand Server from primary. secondary not present');
            return undef,&VRTS::Enclosure::Discover::CONNECT_FAILED;
        }
        # Make secondary as primary for further run
        $ch = $ch1;
    }
    else
    {
        # Even if primary is working fine, if user has provided secondary
        # Try if it is working fine.
        if (defined ($ch1) && $ch1 ne "")
        {
            my $out1 = _get_array_capabilities($ch1,$id,$server,$port,$user,$pass);
            if ($out1 eq "" || $out1 =~ /Server is not/)
            {
                 VRTS::Logger::log('error', 'Unable to contact HiCommand Server secondary');
                 return undef,&VRTS::Enclosure::Discover::SEC_CONNECT_FAILED;
            }
        }
    }
    if( ($out1 ne "") && ($out1 =~ /Unauthorized/) ) 
    {
        VRTS::Logger::log('error', "Access denied. Error returned [$out1]");
        return undef,&VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
    }

    if ($out1 =~ /The logged-in user does not have the permissions for using Device Manager functions/) {
        VRTS::Logger::log('error', "Insufficient privileges. Error returned [$out1]");
        return undef,&VRTS::Enclosure::Discover::SECURITY_ROLE_NOT_PRIVILEGE;
    }

    my $xs = XML::Simple->new(keyattr => []);
    my $xml  = eval{$xs->XMLin($out1)};
    if ($@) {
      $api_version = "2.0";
    } else {
      $api_version = $xml->{Response}->{ResultList}->{ServerInfo}->{serverVersion};
      $_ = $api_version;
      if(/v(\d+\.\d+\.\d+)/) {
          $api_version = $1;
      } elsif(/v(\d+\.\d+)/) {
        $api_version = $1;
      }
      else {
          # TODO : Add Error and exit
          VRTS::Logger::log('error', 'Not able to find XML Version. Defaulting to 2.0');
          $api_version = "2.0";
      }
    }

next_run:
    my $payload =  "hicmd.probe.payload";
    _create_probe_payload($payload, $api_version, $user, $pass);
    my $out = VRTS::Enclosure::Utils::push($ch, $vardir."/".$payload, "\$VARDIR\/hicmd.probe.payload", $probe_timeout);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            VRTS::Logger::log('error', 'Request timed out due to no activity for timeout period');
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         VRTS::Logger::log('error', 'Error in push');
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    my $script = "hicmd.pl";
    my $url = "$protocol://$server:$port/service/StorageManager";
    _create_script($url,$payload,$id,$user,$pass,$script);
    $out = VRTS::Enclosure::Utils::push_exec($ch, $vardir."/".$script, "\$VARDIR\/hicmd.pl", $probe_timeout);
    unlink($vardir."/".$script);
    unlink($payload);
    unlink($vardir."\/hicmd.probe.payload");

    VRTS::Logger::log('trace', "COMMAND = [$url]"); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            VRTS::Logger::log('error', 'Request timed out due to no activity for timeout period');
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         VRTS::Logger::log('error', 'Error in push_exec');
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    if ($out->[1] eq "") {
      return undef, &VRTS::Enclosure::Discover::NO_ARRAYS_DISCOVERED;
    }
  
    my $out_file = "$vardir/hicmd.probe.out";
    open(O, ">$out_file");
    print O "$out->[1]";
    close(O);

    if ($out->[1] =~ /Server is not up/) {
        VRTS::Logger::log('error', 'Unable to receive data from HiCommand...bailing out');
        return undef, &VRTS::Enclosure::Discover::NO_ARRAYS_DISCOVERED;
    }

    my %data;
    my $xs = XML::Simple->new();
    my $xml  = eval{$xs->XMLin($out_file)};
    if ($@) {
        return undef, &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
    }

    my @keys = keys(%{$xml->{Response}->{ResultList}->{StorageArray}});
    my $enchash;
    my $single_array=0;
    foreach my $k (@keys)
    {
        $single_array = 1 if ($k eq "capacityInGB");
        $single_array = 1 if ($k eq "name");
        $single_array = 1 if ($k eq "description");
        $single_array = 1 if ($k eq "arrayFamily");
    }
    if ($single_array == 0)
    {
        foreach my $k (@keys) 
        {
            my $d = $xml->{Response}->{ResultList}->{StorageArray}->{$k};
            my %data;
            my $vdid = %{$d}->{'objectID'};
            my $objid = %{$d}->{'objectID'};

            $data{'serial'} = %{$d}->{'serialNumber'};
            if (defined %{$d}->{'displayArrayType'}) {
              $data{'model'} = %{$d}->{'displayArrayType'};
            } else {
              $data{'model'} = "-";
            }
            $data{'vendor'} = "HITACHI";
            $data{'product'} = %{$d}->{'productName'};
            $data{'displayname'} = $k;
            $data{'api_version'} = $api_version;
            $data{'VDID'} = $vdid;
            $enchash->{$objid} = \%data;
        }
    }
    else
    {
        my $d = $xml->{Response}->{ResultList}->{StorageArray};
        my %data;
        my $vdid = %{$d}->{'objectID'};
        my $objid = %{$d}->{'objectID'};
        $data{'serial'} = %{$d}->{'serialNumber'};
        if (defined %{$d}->{'displayArrayType'}) {
              $data{'model'} = %{$d}->{'displayArrayType'};
        } else {
            $data{'model'} = "-";
        }
        $data{'vendor'} = "HITACHI";
        $data{'product'} = %{$d}->{'productName'};
        $data{'displayname'} = %{$d}->{'name'};
        $data{'api_version'} = $api_version;
        $data{'VDID'} = $vdid;
        $enchash->{$objid} = \%data;
    }
    return $enchash;

}

=pod
sub refresh_device_manager
{
    my $self = shift;
    my $params = shift;

    $ch = $params->{"control-host"};
    $ch1 = $params->{"secondary-ch"};
    my $server = $params->{"server"};
    my $port = $params->{"port"};
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    my $product = $params->{"product"};
    my $api_version = $params->{"api-version"};
    if ($api_version !~ /\./)
    {
        $api_version = $api_version . "\.0";
    }


    my $payload =  "$vardir/refresh_$$.inp";
    _create_refresh_payload($payload,$api_version,$product,$server,$user,$pass);
    my $out = VRTS::Enclosure::Utils::push($ch, $payload, $payload);
    if ($out->[0] != 0)
    {
         VRTS::Logger::log('error', 'Error in push');
         return &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    my $script = "hicmd_refresh.pl";
    my $url = "$protocol://$server:$port/service/StorageManager";
    _create_script($url,$payload,$user,$pass,$script);
    $out = VRTS::Enclosure::Utils::push_exec($ch, $vardir."/".$script, $script);
    unlink($vardir."/".$script);
    if ($out->[0] != 0)
    {
         VRTS::Logger::log('error', 'Error in push_exec');
         return &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    unlink($payload);
    if ($out->[1] ne "")
    {
        my $xs = XML::Simple->new(keyattr => []);
        my $xml  = eval{$xs->XMLin($out->[1])};
        if ($@) {
          return &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
        }
        my $status = $xml->{Response}->{EchoCommand}->{status};
        my $ret = $xml->{Response}->{EchoCommand}->{result};
        my $error = $xml->{Response}->{Error}->{description};
        if ($ret eq "0")
        {
            return 0;
        }
        else
        {
            return &VRTS::Enclosure::Discover::CONNECT_FAILED;
        }
    } 
    else
    {
        return &VRTS::Enclosure::Discover::CONNECT_FAILED;
    }

}

sub get_enclosure_details
{
}
=cut

sub _get_array_capabilities
{
    my $ch = shift;
    my $id = shift;
    my $server = shift;
    my $port = shift;
    my $user = shift;
    my $pass = shift;

    my $payload =  "xmlapi.inp";
    _create_capability_payload($payload, $user, $pass);

    my $out = VRTS::Enclosure::Utils::push($ch, $vardir."/".$payload, "\$VARDIR\/xmlapi.inp", $probe_timeout);
    if ($out->[0] != 0)
    {
         unlink($payload);
         return $out->[0];
    }

    my $script = "hicmd1.pl";
    my $url = "$protocol://$server:$port/service/StorageManager";
    _create_script($url,$payload,$id,$user,$pass,$script);

    $out = VRTS::Enclosure::Utils::push_exec($ch, $vardir."/".$script, "\$VARDIR/hicmd1.pl", $probe_timeout);
    unlink($vardir."/".$script);
    unlink($payload);
    unlink($vardir."/xmlapi.inp");
    unlink($vardir."\/xmlapi.inp");

    VRTS::Logger::log('trace', "COMMAND = [$url]"); 
    VRTS::Logger::log('trace', "OUTPUT = [$out->[1]]"); 

    if ($out->[0] != 0)
    {
        return $out->[0];
    }

    
    return $out->[1];
}

sub _create_script
{
    my ($url, $hicmd_payload, $id, $user, $pass, $script) = @_;

    $hicmd_payload =~ s/\\/\\\\/g;    
    my $script = File::Spec->catfile($vardir,$script);
    open(my $fh, '>', $script) || die "$@: can't open $script for reading: $!";
    binmode $fh;
    print $fh <<EOF;
#!/opt/VRTSsfmh/bin/perl
BEGIN { \@INC = ('/opt/VRTSsfmh/lib/modules') if ( \$^O !~ /Win32/i ) }
require VRTS::Paths;
require VRTS::Util;
my \$xprtlc = VRTS::Paths::get_path('xprtlc');
my \$vardir = VRTS::Paths::get_path('AppDir');
my \$hipayload = File::Spec->catfile(\$vardir,"$hicmd_payload");
my \$payload = File::Spec->catfile(\$vardir,'hixmlapi.inp');

my (\$rc, \$pass) = VRTS::Util::decrypt_encoded_passwd('$pass', '$id');
if (\$rc) {
    print "decrypt_encoded_passwd failed with error code = [\$rc]";
    exit \$rc;
}
open(my \$fhr, '<', \$hipayload) || die "\$@: can't open \$hipayload for reading: \$!";
open(my \$fhw, '>', \$payload)   || die "\$@: can't open \$payload for writing: \$!";
#Replace encrypted password with decrypted password in payload.
while(my \$line = <\$fhr>)
{
    if(\$line =~ /Password:/)
    {
        \$line =  "Password:\$pass\\n";
    }
    print \$fhw \$line;
}
close(\$fhr);
close(\$fhw);
unlink(\$hipayload);

if ( \$^O =~ /Win32/i ) {
    my \$bindir = VRTS::Paths::get_path('MHBin');
    \$xprtlc =  "\$bindir\\\\xprtlc.exe";
    my \$payload1 = "\$payload".'1';
    open(my \$fh1, '<', \$payload);
    open(my \$fh2, '>', \$payload1);
    if (defined(\$fh1) && defined(\$fh2)) {
        binmode \$fh2; 
        my \$str;
        while(<\$fh1>) {
            \$str .= \$_;
        }
        \$str =~ s/\\\\n/\\\\r\\\\n/g;
        print \$fh2 "\$str";
        close(\$fh2);
        close(\$fh1);
    }
    \$pass = VRTS::Util::esc_special_chars(\$pass);
    system(\$xprtlc, '-j', \$payload1, '-l', '$url', '-u', '$user', '-p', \$pass);
    unlink(\$payload1);
} else {
    system(\$xprtlc, '-j', \$payload, '-l', '$url', '-u', '$user', '-p', \$pass);
}
unlink(\$payload);
EOF
    close($fh);
}

sub _create_probe_payload
{
    my ($payload_file, $api_version, $user, $pass) = @_; 
    $payload_file = $vardir . '/' . $payload_file;
    my $r = open(my $fh, '>', $payload_file);
    if (not defined($fh)) {
        VRTS::Logger::log('error', "Unable to open file $payload_file for writing");
        exit $r;
    }

    binmode $fh;
    print $fh <<EOF;
Content-Type: text/xml
Accept: */*
User-Agent: $user_agent
Username:$user
Password:$pass
Connection: close

<?xml version="1.0" encoding="UTF-8"?>
<HiCommandServerMessage>
 <APIInfo version="$api_version"/>
  <Request>
    <StorageManager>
      <Get target="StorageArray" option="all">
         <StorageArray>
        </StorageArray>
      </Get>
    </StorageManager>
  </Request>
</HiCommandServerMessage>
EOF
    close($fh);
}

sub _create_capability_payload
{
    my ($payload_file, $user, $pass) = @_; 
    my $payload_file = File::Spec->catfile($vardir,$payload_file);


    my $r = open(my $fh, '>', $payload_file);
    if (not defined($fh)) {
        VRTS::Logger::log('error', "Unable to open file $payload_file for writing");
        exit $r;
    }

    binmode $fh;
    print $fh <<EOF;
Content-Type: text/xml
Accept: */*
User-Agent: $user_agent
Username:$user
Password:$pass
Connection: close

<?xml version="1.0" encoding="UTF-8"?>
<HiCommandServerMessage>
 <APIInfo version="2.0"/>
    <Request>
        <ServerAdmin>
            <Get target="ServerInfo"/>
        </ServerAdmin>
    </Request>
</HiCommandServerMessage>
EOF
    close($fh);
}

sub _create_refresh_payload
{
    my $payload_file = shift;
    my $api_version = shift;
    my $product = shift;
    my $server = shift;
    my $user = shift;
    my $pass = shift;

    open (C, ">$payload_file");
    print C "Content-Type: text/xml\n";
    print C "Accept: */*\n";
    print C "User-Agent: $user_agent\n";
    print C "Connection: close\n\n";
    print C "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    print C "<HiCommandServerMessage>\n";
    print C " <APIInfo version=\"".$api_version."\" />\n";
    print C "  <Request>\n";
    print C "    <StorageManager>\n";
    print C "      <Add target=\"StorageArray\" >\n";
    print C "         <StorageArray>\n";
    print C "         <CommParameters>\n";
    print C "         <Parameter name=\"arrayFamily\" value=\"$product\"/>\n";
    print C "         <Parameter name=\"ipAddress\" value=\"$server\"/>\n";
    print C "         <Parameter name=\"userID\" value=\"$user\"/>\n";
    print C "         <Parameter name=\"userPassword\" value=\"$pass\"/>\n";
    print C "         </CommParameters>\n";
    print C "        </StorageArray>\n";
    print C "      </Add>\n";
    print C "    </StorageManager>\n";
    print C "  </Request>\n";
    print C " </HiCommandServerMessage>\n";
    close(C);
}

sub provide_capabilities
{
    my $self = shift;

    return \%capabilities;
}


1;
