#$Id$
#$Copyright$

package VRTS::Enclosure::Faults;
use strict;
use Carp;
require Exporter;
use File::Spec;
use Data::Dumper;

use vars qw($VERSION @ISA @EXPORT);

require VRTS::Paths;
require JSON;
require VRTS::Util;
require VRTS::Summary;

@ISA = qw(Exporter);
@EXPORT = qw(compute_tp_faults compute_array_faults);
             
$VERSION = '3.0';
my $delta = 0.01;
my %tp_fault_map = ("SUBS_FAULT_THRESHOLD" => "TP_SUBS_FAULT_THRESHOLD",    # Threshold for subs fault
                        "SUBS_RISK_THRESHOLD" => "TP_SUBS_RISK_THRESHOLD",  # Threshold for subs risk
                        "CONS_RISK_THRESHOLD" => "TP_CONS_RISK_THRESHOLD",  # Threshold for cons risk
                        "CONS_FAULT_THRESHOLD" => "TP_CONS_FAULT_THRESHOLD" # Threshold for cons fault
                    );

my %array_condition_map = ("PROBE_INTERNAL_ERROR" => "MH_NOT_REACHABLE", 
            "MH_NOT_REACHABLE" => "MH_NOT_REACHABLE", 
            "ARRAY_NOT_REACHABLE" => "MH_NOT_REACHABLE");
sub compute_array_faults
{
    my $agent = shift;
    my $condition = shift;
    my $enc_name = shift;
    my $enc_id = shift;

    my $fid = undef;

    if (defined $array_condition_map{$condition})
    {
        $fid = $array_condition_map{$condition};
    }
    else
    {
        $fid = "ARRAY_NOT_REACHABLE"; 
    }

    VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $enc_id,
                    "vrts_ddl_enclosure", $enc_id, $enc_name, $enc_name, $fid);
}

sub compute_tp_faults
{
    my $agent = shift;
    my $param = shift;
    my ($tp_fault_data, $tp_name, $tp_id, $pct_consumed, $pct_subs, $enc_vdid, $enc_name, $tp_tuneables,
         $optimized_disc, $pct_consumed_old, $pct_subs_old);

    # Previous versions will send multiple argumets while newer will accept hash ref.
    # Making compatible with previous versions.
    if(ref $param eq 'HASH') {
        $tp_fault_data = $param;
        $tp_name = $tp_fault_data->{tp_name};
        $tp_id = $tp_fault_data->{tp_id};
        $pct_consumed = $tp_fault_data->{pct_consumed};
        $pct_subs = $tp_fault_data->{pct_subs};
        $enc_vdid = $tp_fault_data->{enc_vdid};
        $enc_name = $tp_fault_data->{enc_name};
        $tp_tuneables = $tp_fault_data->{tp_tuneables};
        $optimized_disc = $tp_fault_data->{opt_disc};
        $pct_consumed_old = $tp_fault_data->{pct_consumed_old};
        $pct_subs_old = $tp_fault_data->{pct_subs_old};
    } else {
        $tp_name = $param;
        $tp_id = shift;
        $pct_consumed = shift;
        $pct_subs = shift;
        $enc_vdid = shift;
        $enc_name = shift;
        $tp_tuneables = shift;
    }

    my %tune_ables;
    my $ret = 1;
    my $fault_tb;
    my @alert_object = ();

    if(defined $optimized_disc and $optimized_disc == 1)
    {
        $fault_tb = $agent->gettable("FAULT_OBJ");
        VRTS::Logger::log('debug', "compute_tp_faults::This is optimized TP[$tp_id] monitoring");
    }
    if (not defined $tp_tuneables->{$tp_id})
    {
        # We need to work with default value for fault consumption which is setup at 90%.
        $tune_ables{SUBS_FAULT_THRESHOLD} = -1;
        $tune_ables{SUBS_RISK_THRESHOLD} = -1;
        $tune_ables{CONS_FAULT_THRESHOLD} = 90;
        $tune_ables{CONS_RISK_THRESHOLD} = -1;
        # No TP tuneables defined, don't report fault/alert.
        VRTS::Logger::log('debug', "No TP tuneables defined for [$tp_id].");
        $ret = 1;
    }
    else
    {
        %tune_ables = %{$tp_tuneables->{$tp_id}};
    }

    #Delete the faults for subscribed capacity if no more valid
    if (($tune_ables{SUBS_RISK_THRESHOLD} > 0 and $pct_subs < $tune_ables{SUBS_RISK_THRESHOLD}) or
        ($tune_ables{SUBS_FAULT_THRESHOLD} > 0 and $pct_subs < $tune_ables{SUBS_FAULT_THRESHOLD}))
    {
        if ($pct_subs != 0)
        {
            if(defined $optimized_disc and $optimized_disc == 1)
            {
                # Check if there is already a reported alert for this thin pool.
                # This alert in no more valid. Need to remove this alert.
                my $alert_str = "SUBS_RISK_THRESHOLD";
                my @tmpcols = (&VRTS::Constants::FAULT_OBJ::OBJ_ID, &VRTS::Constants::FAULT_OBJ::FID);
                my @tmpvals = ($tp_id, $tp_fault_map{$alert_str});
                my $fault_row = $fault_tb->getrowwithvaluez_r(\@tmpcols, \@tmpvals);
                if (not defined $fault_row)
                {
                    $alert_str = "SUBS_FAULT_THRESHOLD";
                    @tmpvals = ($tp_id, $tp_fault_map{$alert_str});
                    $fault_row = $fault_tb->getrowwithvaluez_r(\@tmpcols, \@tmpvals);
                }       

                if(defined $fault_row and (abs($pct_subs - $pct_subs_old) > $delta))
                {
                    VRTS::Logger::log('debug', "compute_tp_faults:Alert was raised earlier for $alert_str. Need to clear since no more valid.");
                    $ret = 0;
                }
            }
        }
    }

    #Delete the faults for consumed capacity if no more valid
    if (($tune_ables{CONS_RISK_THRESHOLD} > 0 and $pct_consumed < $tune_ables{CONS_RISK_THRESHOLD}) or
        ($tune_ables{CONS_FAULT_THRESHOLD} > 0 and $pct_consumed < $tune_ables{CONS_FAULT_THRESHOLD}))
    {
        if ($pct_consumed != 0)
        {
            if(defined $optimized_disc and $optimized_disc == 1)
            {
                # Check if there is already a reported alert for this thin pool on this condition.
                # This alert in no more valid. Need to remove this alert.
                my $alert_str = "CONS_RISK_THRESHOLD";
                my @tmpcols = (&VRTS::Constants::FAULT_OBJ::OBJ_ID, &VRTS::Constants::FAULT_OBJ::FID);
                my @tmpvals = ($tp_id, $tp_fault_map{$alert_str});
                my $fault_row = $fault_tb->getrowwithvaluez_r(\@tmpcols, \@tmpvals);
                if (not defined $fault_row)
                {
                    $alert_str = "CONS_FAULT_THRESHOLD";
                    @tmpvals = ($tp_id, $tp_fault_map{$alert_str});
                    $fault_row = $fault_tb->getrowwithvaluez_r(\@tmpcols, \@tmpvals);
                }       

                if(defined $fault_row and (abs($pct_consumed - $pct_consumed_old) > $delta))
                {
                    VRTS::Logger::log('debug', "compute_tp_faults:Alert was raised earlier for $alert_str. Need to clear since no more valid.");
                    $ret = 0;
                }
            }
        }
    }

    if ($tune_ables{SUBS_FAULT_THRESHOLD} > 0 and $pct_subs > $tune_ables{SUBS_FAULT_THRESHOLD})
    {
        if ($pct_subs != 0)
        {
            if(defined $optimized_disc and $optimized_disc == 1)
            {
                # Check if there is already a reported alert for this thin pool and the
                # values are same.
                # If yes, then no need to generate alert for it again.
                $ret = 0;
                my @tmpcols = (&VRTS::Constants::FAULT_OBJ::OBJ_ID, &VRTS::Constants::FAULT_OBJ::FID);
                my @tmpvals = ($tp_id, $tp_fault_map{SUBS_FAULT_THRESHOLD});
                my $fault_row = $fault_tb->getrowwithvaluez_r(\@tmpcols, \@tmpvals);

                if(defined $fault_row and (abs($pct_subs - $pct_subs_old) < $delta))
                {
                    VRTS::Logger::log('debug', "compute_tp_faults:Alert was raised earlier for SUBS_FAULT_THRESHOLD.");
                    $ret = 1;
                }
            } else {
                VRTS::Summary->generate($agent, "vrts_array_thinpool", $tp_id,
                        "vrts_array_thinpool", $tp_id, $tp_name, $tp_name, $tp_fault_map{SUBS_FAULT_THRESHOLD},
                        undef, undef, undef, undef, undef, undef, $enc_name,$enc_vdid,"vrts_ddl_enclosure");

                # Propogate the risk condition to the enclosure level as well...
                VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $enc_vdid, "vrts_array_thinpool",
                        $tp_id, $tp_name, $enc_name, $tp_fault_map{SUBS_FAULT_THRESHOLD},
                        $tp_id, "vrts_array_thinpool", $tp_fault_map{SUBS_FAULT_THRESHOLD});
                push(@alert_object,'SUBS_FAULT_THRESHOLD');
            }
        }
    }
    elsif ($tune_ables{SUBS_RISK_THRESHOLD} > 0 and $pct_subs > $tune_ables{SUBS_RISK_THRESHOLD})
    {
        if ($pct_subs != 0)
        {
            if(defined $optimized_disc and $optimized_disc == 1)
            {
                # Check if there is already a reported alert for this thin pool.
                # If yes, then no need to generate alert for it again.
                $ret = 0;
                my @tmpcols = (&VRTS::Constants::FAULT_OBJ::OBJ_ID, &VRTS::Constants::FAULT_OBJ::FID);
                my @tmpvals = ($tp_id, $tp_fault_map{SUBS_RISK_THRESHOLD});
                my $fault_row = $fault_tb->getrowwithvaluez_r(\@tmpcols, \@tmpvals);
                if(defined $fault_row and (abs($pct_subs - $pct_subs_old) < $delta))
                {
                    VRTS::Logger::log('debug', "compute_tp_faults:Alert was raised earlier for SUBS_RISK_THRESHOLD.");
                    $ret = 1;
                }
            } else {
                VRTS::Summary->generate($agent, "vrts_array_thinpool", $tp_id,
                        "vrts_array_thinpool", $tp_id, $tp_name, $tp_name, $tp_fault_map{SUBS_RISK_THRESHOLD},
                        undef, undef, undef, undef, undef, undef, $enc_name,$enc_vdid,"vrts_ddl_enclosure");

                # Propogate the risk condition to the enclosure level as well...
                VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $enc_vdid, "vrts_array_thinpool",
                        $tp_id, $tp_name, $enc_name, $tp_fault_map{SUBS_RISK_THRESHOLD},
                        $tp_id, "vrts_array_thinpool", $tp_fault_map{SUBS_RISK_THRESHOLD});
                push(@alert_object, 'SUBS_RISK_THRESHOLD');
            }
        }
    }

    # Now lets tackle the consumption related conditions
    if ($tune_ables{CONS_FAULT_THRESHOLD} > 0 and $pct_consumed > $tune_ables{CONS_FAULT_THRESHOLD})
    {
        if ($pct_consumed != 0)
        {
            if(defined $optimized_disc and $optimized_disc == 1)
            {
                # Check if there is already a reported alert for this thin pool.
                # If yes, then no need to generate alert for it again.
                # Reset the return value if it was changed by subscribed alerts.
                $ret = 0;
                my @tmpcols = (&VRTS::Constants::FAULT_OBJ::OBJ_ID, &VRTS::Constants::FAULT_OBJ::FID);
                my @tmpvals = ($tp_id, $tp_fault_map{CONS_FAULT_THRESHOLD});
                my $fault_row = $fault_tb->getrowwithvaluez_r(\@tmpcols, \@tmpvals);
                if(defined $fault_row and (abs($pct_consumed - $pct_consumed_old) < $delta))
                {
                    VRTS::Logger::log('debug', "compute_tp_faults: Alert was raised earlier for CONS_FAULT_THRESHOLD.");
                    $ret = 1;
                }
            } else {
                # Fault condition as the consumed size is sitting at 100%
                VRTS::Summary->generate($agent, "vrts_array_thinpool", $tp_id,
                        "vrts_array_thinpool", $tp_id, $tp_name, $tp_name, $tp_fault_map{CONS_FAULT_THRESHOLD},
                        undef, undef, undef, undef, undef, undef, $enc_name,$enc_vdid,"vrts_ddl_enclosure");

                # Propogate the risk condition to the enclosure level as well...
                VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $enc_vdid, "vrts_array_thinpool",
                        $tp_id, $tp_name, $enc_name, $tp_fault_map{CONS_FAULT_THRESHOLD},
                        $tp_id, "vrts_array_thinpool", $tp_fault_map{CONS_FAULT_THRESHOLD});
                push(@alert_object, 'CONS_FAULT_THRESHOLD');
            }
        }
    }
    elsif ($tune_ables{CONS_RISK_THRESHOLD} > 0 and $pct_consumed > $tune_ables{CONS_RISK_THRESHOLD})
    {
        if ($pct_consumed != 0)
        {
            if(defined $optimized_disc and $optimized_disc == 1)
            {
                # Check if there is already a reported alert for this thin pool.
                # If yes, then no need to generate alert for it again.
                $ret = 0;
                my @tmpcols = (&VRTS::Constants::FAULT_OBJ::OBJ_ID, &VRTS::Constants::FAULT_OBJ::FID);
                my @tmpvals = ($tp_id, $tp_fault_map{CONS_RISK_THRESHOLD});
                my $fault_row = $fault_tb->getrowwithvaluez_r(\@tmpcols, \@tmpvals);
                if(defined $fault_row and (abs($pct_consumed - $pct_consumed_old) < $delta))
                {
                    VRTS::Logger::log('debug', "compute_tp_faults: Alert was raised earlier for CONS_RISK_THRESHOLD.");
                    $ret = 1;
                }
            } else {
                # Fault condition as the consumed size is sitting at 100%
                VRTS::Summary->generate($agent, "vrts_array_thinpool", $tp_id,
                        "vrts_array_thinpool", $tp_id, $tp_name, $tp_name, $tp_fault_map{CONS_RISK_THRESHOLD},
                        undef, undef, undef, undef, undef, undef, $enc_name,$enc_vdid,"vrts_ddl_enclosure");

                # Propogate the risk condition to the enclosure level as well...
                VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $enc_vdid, "vrts_array_thinpool",
                        $tp_id, $tp_name, $enc_name, $tp_fault_map{CONS_RISK_THRESHOLD},
                        $tp_id, "vrts_array_thinpool", $tp_fault_map{CONS_RISK_THRESHOLD});
                push(@alert_object,'CONS_RISK_THRESHOLD');
            }
        }
    }
    # IF there is some unified array defined then raise the alert at that level
    # too.
    if(defined $tp_fault_data->{unified_vdid} and $tp_fault_data->{unified_vdid} ne '' and scalar(@alert_object) != 0) 
    {
         generate_additional_faults($agent, $tp_fault_data, \@alert_object);
    }

    VRTS::Logger::log('debug', "compute_tp_faults:returning [$ret]");
    return $ret;
}

sub generate_additional_faults()
{
    my $agent = shift;
    my $tp_fault_data = shift;
    my $alert_object = shift;
    foreach my $alert(@$alert_object) {
        VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $tp_fault_data->{unified_vdid}, "vrts_array_thinpool",
                            $tp_fault_data->{tp_id}, $tp_fault_data->{tp_name}, 
                            $tp_fault_data->{unified_name}, $tp_fault_map{$alert},
                            $tp_fault_data->{tp_id}, "vrts_array_thinpool", $tp_fault_map{$alert});
    }
}
1;
