#$Id$
#$Copyright$

package VRTS::Enclosure::VDID;
use strict;
use Carp;
require Exporter;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $TO_DEFAULT);

@ISA = qw(Exporter);
@EXPORT = qw(new initialize switch get_enclosure_vdid get_disk_vdid);
$VERSION = '3.0';

$TO_DEFAULT = 0;

sub new 
{
    my $class = shift;
    my $self = {};

    # configuration options
    $self->{CONFIGOPTIONS} = {};
    $self->{PLUGIN} = {};
    $self->{HAVE_PLUGIN} = 0;

    bless ($self, $class);
    return $self;
}

sub initialize 
{
    my $self = shift;
    my $vendor = shift;
    my $to_default = shift;

    $TO_DEFAULT = 1 if (defined $to_default and $to_default == 1);
    
    # Based on the vendor information that is passed, 
    #  setup the lower level module.
    my $module_name = "VRTS::Enclosure::VDID::".$vendor;
    if (eval "require $module_name")
    {
        my $vendor_handle  = &{\&{"${module_name}::initialize"}};
        $self->{HAVE_PLUGIN} = 1;
        $self->{PLUGIN} = $vendor_handle;
    }
    else
    {
        croak "$@ module [$vendor] for [$module_name] not available\n";
    }
}

sub switch
{
    my $self = shift;
    my $vendor_class = shift;
}

sub get_enclosure_vdid
{
    my ($self, $vendor, $product, $cabsno) = @_;

    return undef if (not defined $self);
    return undef if ($self->{HAVE_PLUGIN} == 0 and $TO_DEFAULT == 0);

    if (defined $self->{PLUGIN})
    {
        return $self->{PLUGIN}->get_enclosure_vdid($vendor, $product, $cabsno);
    }
    elsif ($TO_DEFAULT == 1)
    {
        # Route the call to the default implementation....
    }

    return undef;
}

sub get_disk_vdid
{
    my ($self, $vendor, $product, $cabsno, $serial, $is_pdev, $pdev_sno) = @_;

    return undef if (not defined $self);
    return undef if ($self->{HAVE_PLUGIN} == 0 and $TO_DEFAULT == 0);

    if (defined $self->{PLUGIN})
    {
        return $self->{PLUGIN}->get_disk_vdid($vendor, $product, $cabsno, $serial, $is_pdev, $pdev_sno);
    }
    elsif ($TO_DEFAULT == 1)
    {
        # Route the call to the default implementation....
    }

    return undef;
}

1;
