#$Id$
#$copyright$

package VRTS::Enclosure::VDID::Hitachi;
use strict;
use Carp;
require Exporter;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $TO_DEFAULT);

use constant MAX_LDEV_PER_CU => 256;

sub new
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::VDID::Hitachi;
    return $module;

}


sub get_disk_vdid
{
    my ($self, $vendor, $product, $cabsno, $serial, $is_pdev, $devnum, $vdid) = (@_, undef);

    $product =~ s/HDS//g;
    $vdid = generate_ldev_vdid ($vendor,$product,$serial,$devnum);
    return $vdid;
}

#############
# Logic:
#
# 1. Get Disk CU Number
# 2. Get LDEV Number
# 3. Construct VDID
###################

sub
generate_ldev_vdid
{
    my ($vendor, $product, $serial, $devnum) = @_;

    my $cu = _get_cu($devnum);
    my $ldev = _get_ldev_id($devnum);
    if ($product =~ m/^95/)
    {
        #It is a 9500V model, so strip the serialnumber of 6xxx and use just last 4 bytes from the 8 byte serialnumber.
        $serial= substr $serial,4,4;
    }
    # Now Strip off any leading zeroes.
    $serial=~ s/^0+//;
    my $vdid = $vendor . '_' . $product . '_' . $serial . '_' . $cu . '_' . $ldev;
    return $vdid;
}


# Compute and return CU id in hex.
sub
_get_cu()
{
    my $devnum = shift;
    my $cu = int($devnum / MAX_LDEV_PER_CU);
    my $hex_cu = sprintf('%x', $cu);
    return $hex_cu;
}

# Compute and return LDEV id in hex.
sub
_get_ldev_id()
{
    my $devnum = shift;
    my $id = $devnum % MAX_LDEV_PER_CU;
    my $hex_id = sprintf('%x', $id);
    return $hex_id;
}

sub
get_enclosure_vdid
{
    my ($self, $vendor, $product, $cabsno, $isHP) = (@_, 0);
    my ($vdid, $model);
    if ($product =~ /^XP/)
    {
        $isHP = 1;
        $vdid = 'HP_';
    }
    elsif($product =~ /SUN/)
    {
        $vdid = 'SUN_';
    }
    else
    {
        $vdid = 'HITACHI_';
    }

    if ($isHP == 1)
    {
        $model = $product;
        #HBAAPP uses XP12000, even if the model is 501 which is the HP XP10000, so put XP12000 in the vdid
        #do the same for XP20000 as well
        $model = 'XP12000' if ($model eq 'XP10000' || $model eq 'XP20000');
        $vdid .= $model . '_';
      
    }
    else
    {
         #For USP or AMS/WMS we use the family
         $product =~ s/HDS//g;
         if ($product =~ /WMS|D800/) {
             # Making the UID same as AMS, for in band correlation
             $vdid .= 'AMS_';
         } else {
             $vdid .= $product . '_';
         }
    }
    my $sno = _get_serial_no($product,$cabsno);
    return undef if ($sno eq '');
    $vdid .= $sno;
    return $vdid;
}

sub
_get_serial_no
{
    my ($product, $sno) = @_;

    if ($product =~ m/^95/)
    {
        #It is a 9500V model, so strip the serialnumber of 6xxx and use just last 4 bytes from the 8 byte serialnumber.
        $sno = substr $sno,4,4;
    }
    # Now Strip off any leading zeroes.
    $sno =~ s/^0+//; 
    return $sno;
}

1;
