#$Id$
#$Copyright$

package VRTS::Enclosure::SIPlugin::Util;
use strict;

require Exporter;
require VRTS::Enclosure::Utils;
require File::Spec;
require File::Temp;
require File::Basename;
require VRTS::Enclosure::SIPlugin::Catalog;

use vars qw($VERSION @ISA @EXPORT );
$VERSION = '1.0';
@ISA = qw(Exporter);
@EXPORT = qw(exec_cmd get_error get_error_res_id get_config_by_vendor_id setup_discovery_scripts 
             get_ldevid_from_name get_encl_from_alt_id get_ldevid_from_alt_id get_enclid_from_name get_error_res_map get_all_plugin_configs);

my $vardir = VRTS::Paths::get_path("AppDir");
my %msg_map = (
   &VRTS::Enclosure::SIPlugin::Catalog::VENDOR_COMP_NOT_FOUND => 
      "The required vendor components not found at the specified location",
   &VRTS::Enclosure::SIPlugin::Catalog::NO_ARRAYS_DISCOVERED => 
      "No enclosure(s) discovered via the specified discovery mechanism",
   &VRTS::Enclosure::SIPlugin::Catalog::PROBE_INTERNAL_ERROR => 
      "Internal error encountered during enclosure discovery",
   &VRTS::Enclosure::SIPlugin::Catalog::MH_NOT_REACHABLE  =>
      "Failed to get results, discovery host may be down",
   &VRTS::Enclosure::SIPlugin::Catalog::INVALID_FIELD_VALUE =>
      "Invalid value specified for one or more field(s)",
   &VRTS::Enclosure::SIPlugin::Catalog::NO_OBJECTS_DISCOVERED =>
      "No objects discovered",
   &VRTS::Enclosure::SIPlugin::Catalog::FAILED_TO_OPEN_DATA_FILE =>
      "Failed to open disovery data file",
   &VRTS::Enclosure::SIPlugin::Catalog::DATA_FILE_NOT_FOUND =>
      "Discovery data file not found",
   &VRTS::Enclosure::SIPlugin::Catalog::DSC_SCRIPT_SETUP_FAILED =>
      "Failed to install discovery scripts on discovery host",
   &VRTS::Enclosure::SIPlugin::Catalog::DSC_SCRIPT_NOT_FOUND =>
      "Discovery script not found",
   &VRTS::Enclosure::SIPlugin::Catalog::DSC_CMD_FAILED =>
      "Discovery command failed",
   &VRTS::Enclosure::SIPlugin::Catalog::FAILED_TO_EXEC_DSC_CMD =>
      "Failed to execute the the discovery command" 
              );

my %resource_id_map = (
   &VRTS::Enclosure::SIPlugin::Catalog::VENDOR_COMP_NOT_FOUND =>
      "VENDOR_COMP_NOT_FOUND",
   &VRTS::Enclosure::SIPlugin::Catalog::NO_ARRAYS_DISCOVERED =>
      "NO_ARRAYS_DISCOVERED",
   &VRTS::Enclosure::SIPlugin::Catalog::PROBE_INTERNAL_ERROR =>
      "PROBE_INTERNAL_ERROR",
   &VRTS::Enclosure::SIPlugin::Catalog::MH_NOT_REACHABLE  =>
      "MH_NOT_REACHABLE",
   &VRTS::Enclosure::SIPlugin::Catalog::INVALID_FIELD_VALUE =>
      "INVALID_FIELD_VALUE",
   &VRTS::Enclosure::SIPlugin::Catalog::NO_OBJECTS_DISCOVERED =>
      "NO_OBJECTS_DISCOVERED",
   &VRTS::Enclosure::SIPlugin::Catalog::DSC_SCRIPT_SETUP_FAILED =>
      "DSC_SCRIPT_SETUP_FAILED",
   &VRTS::Enclosure::SIPlugin::Catalog::FAILED_TO_OPEN_DATA_FILE =>
      "FAILED_TO_OPEN_DATA_FILE",
   &VRTS::Enclosure::SIPlugin::Catalog::DATA_FILE_NOT_FOUND =>
      "DATA_FILE_NOT_FOUND",
   &VRTS::Enclosure::SIPlugin::Catalog::DSC_CMD_FAILED =>
      "DSC_CMD_FAILED",
   &VRTS::Enclosure::SIPlugin::Catalog::FAILED_TO_EXEC_DSC_CMD =>
      "FAILED_TO_EXEC_DSC_CMD",
   &VRTS::Enclosure::SIPlugin::Catalog::DSC_SCRIPT_NOT_FOUND =>
      "DSC_SCRIPT_NOT_FOUND",
              );


# Description:
#    Method to push and execute a script. The script executes the given command on the
#    specified host.
# Input:
#    ch - host where the script has to be pushed
#    cli_path - the path to the given command
#    cmd - the command to bundle in the script
#    args - Args to be passed to the command
# Output:
#    out - this a refrence to an array where 
#          $out->[0] is the return code that indicates if the push of the
#            of the script succeeded.0 is success. nonzeo is error.
#          $out->[1] is the file handle to the command o/p
#      
sub exec_cmd
{
    my $ch = shift;
    my $context = shift;

    my $family = $context->{FAMILY};
    my $cmd = $context->{CMD};
    my $cmd_loc = $context->{CMD_LOC};
    my $cmd_line = $context->{CMD_LINE};
    my $encl_vdid = $context->{encl_vdid};
    my $out;

    VRTS::Logger::log('debug', "cmd loc $cmd_loc.cmd is $cmd");

    if ($context->{'execution-ctxt'} eq "sdktest" && $context->{'test-source'} == 1)
    {
        $cmd_line =~ /--list ([\S]+)/;
        $cmd_line = File::Spec->catfile($cmd_loc, $1.".txt"); 
        VRTS::Logger::log('debug', "command data file $cmd_line");
        if (!-f $cmd_line)
        {
            VRTS::Logger::log('error', "data file $cmd_line not found");
            $out->[0] = &VRTS::Enclosure::SIPlugin::Catalog::DATA_FILE_NOT_FOUND;
            return $out;
        }
        if ( open (FH, $cmd_line) == 0)
        {
            VRTS::Logger::log('error', "Failed to open data file $cmd_line");
            $out->[0] = &VRTS::Enclosure::SIPlugin::Catalog::FAILED_TO_OPEN_DATA_FILE;
            $out->[1] = $!;
            return $out;
        }
        # Return file handle to script o/p and command return code
        $out->[1] = *FH{IO};
        $out->[0] = $?;
        return $out;
    }
  
    my $script = File::Spec->catfile($vardir,"disc_".$family.$$);
    $script .= ".pl";

    my $script_bname = "ddisc_".$family.$$.".pl";

    open(S, ">$script");
    binmode S;
    print S "\#\!\/opt\/VRTSsfmh\/bin\/perl\n".
            "BEGIN { \@INC = (\"/opt/VRTSsfmh/lib/modules\") if ( \$\^O !~ /Win32/i ) };\n\n".
            "require File::Spec;\n".
            "require VRTS::Paths;\n".
            "my \$cmd_line = \"$cmd_line\";\n".
            "my \$install_dir = VRTS::Paths::get_path(\"InstallDir\");\n".
            "my \$app_dir = VRTS::Paths::get_path(\"AppDir\");\n".
            "my \$cmd_name = \"$cmd\";\n".
            "my \$cmd_path = \"$cmd_loc\";\n".
            "my \$cmd_line = \"$cmd_line\";\n".
            "my \$plugin_dir = File::Spec->catfile(\$install_dir,\"etc\", \"SIPlugin\", \"$family\");\n".
             "\$ENV{SI_PLUGIN_DIR} = \$plugin_dir;\n";
    if (defined $encl_vdid)
    {
        print S "my \$plugin_work_dir = File::Spec->catfile(\$app_dir,\"$family\",\"$encl_vdid\");\n".
                "\$ENV{SI_PLUGIN_WORK_DIR} = \$plugin_work_dir;\n";
    }

    if (defined $cmd_loc && $cmd_loc ne "")
    {
        print S "if (\$cmd_path eq \"default\") {\n".
                "   \$cmd_path = \$plugin_dir;\n".
                "   chdir \$cmd_path;\n".
                "   \$cmd_path = File::Spec->catfile(\$cmd_path, \$cmd_name);\n".
                "   \$cmd_line  =~ s/\$cmd_name/\\.\\/\$cmd_name/;\n".
                "}\n".
                "else {\n".
                "   \$cmd_path = File::Spec->catfile(\$cmd_path, \$cmd_name);\n".
                "    \$cmd_line  =~ s/\$cmd_name/\"\$cmd_path\"/;\n".
                "}\n".
                "if (!-f \"\$cmd_path\") {\n".
                " \t print \"\\nCommand not found or not executable: \$cmd_path\\n\";\n".
                " exit(1)\n".
                "}\n";
    }

    if ($cmd =~ /\.pl$/)
    {
        print S "my \$perl_path = File::Spec->catfile(\$install_dir,\"bin\", \"perl\");\n".
                "if ( \$\^O =~ /Win32/i)\n".
                "   {\n".
                "      \$perl_path = File::Spec->catfile(\$install_dir,\"bin\", \"perl.exe\");\n".
                "   }\n".
                "\$cmd_line = \"\\\"\$perl_path\\\" \".\$cmd_line;".
                "\n";
    }
    print S "system(\$cmd_line.\" 2>&1\");\n".
            "if (\$? == -1 || (\$? & 127)) { \n".
            " print \"Failed to execute CLI: $!\";\n".
            " exit (1);\n}".
            "elsif (\$?) { print \"CLI failed with error: \$?\"; exit(1);}\n";
    close(S);
    if ($cmd_line =~ /get-data/)
    {
        $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/$script_bname",undef,undef);
    }
    else{
        $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/$script_bname",undef,1);
    }

    return $out;
}

# Description:
#    Method to push discovery scripts and set them up on the given host.
#    The specified tar.gz file that contains the discovery scripts is pushed to the
#    specified host. The scripts are installed in the default location <VRTSsfmh installdir>/etc/SIPlugin/family
#    if a script by the name checkenv.pl exists in the tar then it is executed.
#    checkenv is expected to check if components to execute the bundled scripts 
#    are present on the host.
# Input:
#    family - family of the plugin
#    ch - host where the discovery scripts have to be setup
#    discovery_scripts - location of the tar file that contains the discovery scripts
#    check_env_args - Args to be passed to the checkenv script
# Output:
#    out -
#          $out->[0] is the return code that indicates if the push and setup of the
#            of the discovery scripts succeeded.0 is success. nonzero is error.
#          $out->[1] is the error message in case push/setup/checkevn fails
#
sub setup_discovery_scripts
{
    my $family = shift;
    my $ch = shift;
    my $discovery_scripts = shift;
    my $check_env_args = shift;
    my $fn = "setup_discovery_scripts";

    my $tmp_name = File::Temp::tempnam($vardir,"siscript");
    my $script_bname = File::Basename::basename($tmp_name);

    my $scripts_tar = $script_bname.".tar.gz";
    my $script = $tmp_name.".pl";
    $script_bname .= ".pl";

    my $src_module_dir = File::Basename::dirname($discovery_scripts);

    my $out = VRTS::Enclosure::Utils::push($ch, $discovery_scripts, "\$VARDIR\/$scripts_tar",undef);
    if ($out->[0] != 0)
    {
        VRTS::Logger::log ('error', "$fn. Failed to push discovery modules on $ch:$out->[1]");
        $out->[0] = &VRTS::Enclosure::SIPlugin::Catalog::MH_NOT_REACHABLE;
        $out->[1] = undef;
        return $out;
    }

    my $tar_location = $out->[1];
    $tar_location =~ s/SUCCESS\n//;
    chomp ($tar_location);

    # fix the tar location if pushed on a windows host
    $tar_location =~ s/\\/\\\\/g;

    VRTS::Logger::log ('debug', "module scripts pushed at location $tar_location");
    my $config_script = File::Spec->catfile ($src_module_dir, "setup-modules.pl");

    open (S, ">$script");
    binmode S;

    open (F, $config_script);
    while (<F>)
    {
        $_ =~ s/##TAR_LOCATION##/$tar_location/;
        $_ =~ s/##ARGS##/$check_env_args/;
        $_ =~ s/##FAMILY##/$family/; 
        print S $_;
    }
    close(F);
    close(S);

    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/$script_bname",undef);
    if ($out->[0] != 0)
    {
        VRTS::Logger::log ('error', "Failed to execute the setup script on $ch: $out->[1]");
        $out->[0] = &VRTS::Enclosure::SIPlugin::Catalog::DSC_SCRIPT_SETUP_FAILED;
        $out->[1] = undef;
    }
    elsif ($out->[1] =~ /Plugin checkenv command failed with error/)
    {
        VRTS::Logger::log ('error', "check_env script failed on $ch: $out->[1]");
        $out->[0] = &VRTS::Enclosure::SIPlugin::Catalog::VENDOR_COMP_NOT_FOUND;
        $out->[1] = undef;
    }
    elsif ($out->[1] =~ /Failed to setup discovery scripts:/)
    {
        VRTS::Logger::log ('error', "Failed to setup discovery scripts on $ch: $out->[1]");
        $out->[0] = &VRTS::Enclosure::SIPlugin::Catalog::DSC_SCRIPT_SETUP_FAILED;
        $out->[1] = undef;
    }
    unlink($script);

    return $out;
}

# Method returns the error message corrosponding to the error code.
# It is intended to return localized messages in the future.
sub get_error
{
    my $msg = shift;
    my $err_code = shift;
    if (defined $err_code && exists $msg_map{$err_code})
    {
        return $msg_map{$err_code};
    }
    return $msg;
}

sub get_error_res_id
{
    my $err_code = shift;
    if (defined $err_code && exists $resource_id_map{$err_code})
    {
        return $resource_id_map{$err_code};
    }
    return $err_code;
}

sub get_error_res_map
{
    return \%resource_id_map;

}

# Description : Method returns the device configuration parameters given a device vdid.
#   It reads the execution context from the model configuration data. 
#   if the context is sdktest then configuration parameters are read from the model configuration data itself.
#   if not then they are fetched from the database.
# Input :
#   objid - vdid of the device
#   config_data - hash containing the model configuration data(typically read from the configuration file)
# Output :
#   config : Hash containing the device configuration data for the specified device 
sub get_config_by_vendor_id
{
    my $agent = shift;
    my $objid = shift;
    my $config_data = shift;

    my $config;
    my $context = $agent->get_context();
    if (defined $context->{'execution-ctxt'} && $context->{'execution-ctxt'} eq "sdktest")
    {
        $config->[0]->{METHOD} = $config_data->{ARRAY_PLUGIN_FAMILY};
        $config->[0]->{CONTROL_HOST} = "localhost";
        foreach my $k (keys %{$config_data->{DEVICE_CONFIG_META}})
        {
            $config->[0]->{$k} = $config_data->{DEVICE_CONFIG_META}->{$k}->{test_value}
                if (defined $config_data->{DEVICE_CONFIG_META}->{$k}->{test_value});
        }
    }
    else
    {
        $config = VRTS::Enclosure::Utils::get_config_by_vendor_id($objid);
    }
    return $config;
}

# Method gives the enclosure id given the enclosure name, product and model
# by quering the VOM DB
sub get_enclid_from_name
{
    my $encl_name = shift;
    my $encl_vendor = shift;
    my $encl_model = shift;

    return undef if (!defined $encl_name || $encl_name eq "" ||
                     !defined $encl_vendor || $encl_vendor eq "" ||
                     !defined $encl_model || $encl_model eq ""); 
    my $sql = "SELECT arr.ID from HABDBSYNC.P_ARRAY_SUBSYSTEM arr, HABDBSYNC.P_CONFIG_VENDOR_MAP cvm,".
              " HABDBSYNC.P_ARRAY_CONFIG_ARRAY_MAPPING acm, HABDBSYNC.P_ARRAY_CONFIG ac".
              " WHERE arr.ID = acm.VENDOR_ID AND acm.ID = ac.ID AND ".
              " ac.METHOD = cvm.METHOD AND cvm.VENDOR = '$encl_vendor' ".
              " AND cvm.MODEL = '$encl_model' AND arr.NAME = '$encl_name';";
    VRTS::Logger::log('debug', 
       "get_enclid_from_name:Fetching encl id from DB[$encl_name,$encl_vendor,$encl_model]");
    my $out = VRTS::Enclosure::Utils::_exec_sql($sql);
    return $out;
}


# Method gives the enclosure id given the enclosure alternate id
# by quering the VOM DB
sub get_encl_from_alt_id
{
    my $encl_alt_id = shift;
    return undef if (!defined $encl_alt_id);

    my $sql = "SELECT arr.ID from HABDBSYNC.P_ARRAY_SUBSYSTEM arr where arr.ALT_ID = '$encl_alt_id';";
    my $out = VRTS::Enclosure::Utils::_exec_sql($sql);
    return $out;
}


# Method gives the ldev id given the enclosure id and ldev name by quering the VOM DB
sub get_ldevid_from_name
{
    my $ldev_name = shift;
    my $encl_id = shift;

    my $sql = "SELECT ID FROM HABDBSYNC.P_ARRAY_LDEV WHERE NAME = '$ldev_name' AND SUBSYS_ID ='$encl_id';";
    my $out = VRTS::Enclosure::Utils::_exec_sql($sql);
    return $out;
}

# Method gives the ldev id given the enclosure id and alternate id for the lun by quering the VOM DB
sub get_ldevid_from_alt_id
{
    my $ldev_alt_id = shift;
    my $encl_id = shift;

    return undef if (!defined $ldev_alt_id || ! defined $encl_id);
    my $sql = "SELECT ID FROM HABDBSYNC.P_ARRAY_LDEV WHERE ALT_ID = '$ldev_alt_id' AND SUBSYS_ID ='$encl_id';";
    my $out = VRTS::Enclosure::Utils::_exec_sql($sql);
    return $out;
}


# given a plugin family, the method returns all array config mapping records
# for that plugin
sub get_all_plugin_configs
{
    my $family = shift;

    my $sql = "select * from habdbsync.P_ARRAY_CONFIG ac, habdbsync.P_ARRAY_CONFIG_ARRAY_MAPPING acm ".
              "where ac.id=acm.id and ac.method = '$family';";
    my $out = VRTS::Enclosure::Utils::_exec_sql($sql);
    return $out;
}

1;
