#!/opt/VRTSsfmh/bin/perl
#$Id$
#$Copyright$

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i);}

use strict;
use VRTS::Paths;
use File::Spec;
use VRTS::Util;
use VRTS::Ini;
use File::Temp;
use JSON;
use VRTS::Logger;
use VRTS::Schedule;
use Getopt::Long;
use Data::Dumper;
use VRTS::AddOnInfo;

my $context = $ENV{context};
if (!defined $context)
{
    $context = "SI";
}
my $config_data;
my $vendor;
my $model;
my $addon_id;
my $app_dir = VRTS::Paths::get_path ("AppDir");
my $addon_dir = $ENV{Store};
my $installdir = VRTS::Paths::get_path ("InstallDir");
my $appdircs = $app_dir;
   $appdircs =~ s/VRTSsfmh/VRTSsfmcs/;
my @cleanup_files;
my $shared_dir = File::Spec->catdir ($app_dir, "shared");

my $perl = File::Spec->catfile ($installdir, "bin", "perl");
my $array_driver = File::Spec->catfile ($installdir, "bin", "array_driver.pl");

my $log_dir = File::Spec->catdir($app_dir,"logs");
my $log = new VRTS::Logger(Module => 'siplugin',
                           Logdir => $log_dir,
                           Level => 'debug',
                           Logfile => 'siplugin-setup.log',
                           UmiString => '');
my $ret;
my %opts;
my ($array_family, $version);
my $agentlet_dir;
my %plugin_bits;
my $si_installed;
my $active_cms;
my $si_ver;
my $discv_method;
my $ha;

GetOptions(\%opts,qw (operation=s));
$opts{operation} = "install" if (!defined $opts{operation});

VRTS::Logger::log ('debug', "operation is $opts{operation}. context is $context");

if ($opts{operation} eq "uninstall")
{
    $ret = get_env();
    goto unsetuperr if ($ret);

    $ret = cleanup();
    goto unsetuperr if ($ret);

    
    VRTS::Logger::log('debug' , "Successfully uninstalled SI plugin for $array_family($version)\n");
    print "\nSuccessfully uninstalled SI plugin for $array_family($version)\n";
    exit(0);

unsetuperr:

    VRTS::Logger::log('debug' , "Failed to uninstall the SI Plugin for $array_family($version)".
     "Refer to siplugin-setup.log for details");
    print STDERR "\nFailed to uninstall the SI Plugin for $array_family($version)".
     "\nRefer to siplugin-setup.log for details";    
    exit(1);
}


$ret = get_env();
goto err if ($ret);

if ($si_installed == 1 && $si_ver =~ /^4\./)
{
     print STDERR "Cannot install SI Plugin as incompatible version of SI Add-on found installed\n"; 
     goto err;
}

if ($context ne "sdktest")
{
    if ($si_installed == 0)
    {
        print STDERR "SI Add-on of the required version is not installed\n";
        goto err;
    }
}

VRTS::Logger::log('debug', "Executing in context $context");
$ret = register_plugin();
if ($ret)
{
    cleanup ();
    goto err; 
}

VRTS::Logger::log('debug' , "Successfully installed SI plugin for $array_family($version)\n");
print "Successfully installed SI plugin for $array_family($version)\n";
exit(0);

err:
    VRTS::Logger::log('error' , "Failed to install SI Plugin for $array_family($version)\n");
    print "Failed to install SI Plugin for $array_family($version)\n";
    exit(1);


# method registers the plugin. Following steps are performed:-
# Plugin bits are copied to target location.
# Plugin family is registered with the configured domains
# Plugin device configuration parameters are registered in the database
sub register_plugin
{
    my $ret = 0;
    my $config_file = File::Spec->catfile ($addon_dir, "plugin_def.conf");
    my $msg;
    my $arr_config_tab = "P_ARRAY_CONFIG";
    my $arr_config_params = "P_ARRAY_CONFIG_PARAMS";
    my $cmd;
    my $out;

    VRTS::Logger::log('debug' , "Registering the SI plugin $array_family. version $version..");

    # Remove stale plugin components if any
    cleanup ();

    if ($context eq "sdktest")
    {
        # generate constant for the tdfs
        chdir $addon_dir;
        $cmd = File::Spec->catfile($installdir,"bin","tdf2constant.pl");
        $cmd = make_command($perl, $cmd, "--dir", File::Spec->catfile($addon_dir, "tdf"));
        $out = system ($cmd);
    }

    $ret = copy_plugin_bits();
    return 1 if ($ret);

    # These are configuration keys which are already present as columns in
    # the P_ARRAY_CONFIG table. the values for these will be stored in the
    # P_ARRAY_CONFIG table and not the P_ARRAY_DEVICE_CONFIG_PARAMS table
    # as the space for them is already accounted for in the table P_ARRAY_CONFIG.
    # Any new configuration keys that get added going ahead will be stored in the params table.
    my %default_config_params =
    (
        "USERNAME" => $arr_config_tab.".USERNAME",
        "DEVICE_MANAGER" => $arr_config_tab.".DEVICE_MANAGER",
        "PASSWD" => $arr_config_tab.".PASSWD",
        "PEMFILE" => $arr_config_tab.".PEMFILE",
        "INSTALL_PATH" => $arr_config_tab.".INSTALL_PATH",
        "PORT" => $arr_config_tab.".PORT"
    );

    my $family_dir = File::Spec->catdir($app_dir, $array_family);
    File::Path::rmtree($family_dir);

    eval ( File::Path::mkpath($family_dir, 0, 0755) );
    if($@)
    {
        $msg = "Cannot mkdir $@";
        VRTS::Logger::log ('error', $msg);
        print STDERR $msg;
        return 1;
    }

    my $family_conf = File::Spec->catdir($family_dir, "plugin_def.conf");
    if ($context eq "sdktest" || $active_cms == 1)
    {
        # Do not setup the plug-in def file on a HA slave.
        # The directory to be shared should be empty on the slave node
        # else the creation of link to the shared location on the slave node fails.
        File::Copy::copy ($config_file, $family_conf);
    }

    return 0 if ($context eq "sdktest");

    $ret = register_family_with_domains();
    return 1 if ($ret);

    return 0 if ($active_cms == 0);

    my $id;
    my $default_value;

    my $sql= "delete from habdbsync.p_array_device_config_meta where model = \'$array_family\' and type = \'ARRAY\';\n";
    foreach my $key (keys %{$config_data->{'DEVICE_CONFIG_META'}})
    {
        $id = $key;
        my $config_key = undef;

        $config_key = $default_config_params{$key} if (exists $default_config_params{$key});
        if (!defined $config_key)
        {
            $config_key = $arr_config_params.".".$key;
        }
        my $type = $config_data->{'DEVICE_CONFIG_META'}->{$key}->{'datatype'};
        my $display_name = $config_data->{'DEVICE_CONFIG_META'}->{$key}->{'display_name'};
        my $is_mandatory = $config_data->{'DEVICE_CONFIG_META'}->{$key}->{'is_mandatory'};
        $default_value = $config_data->{'DEVICE_CONFIG_META'}->{$key}->{'default_value'};

        if (defined $is_mandatory && $is_mandatory == 1)
        {
            $is_mandatory = 1;
        }
        else
        {
            $is_mandatory = 0;
        }
        $sql .= "insert into habdbsync.p_ui_resource(id) values('$id');\n".
                "insert into habdbsync.p_ui_res_bundle(id,locale_id,display) values".
                "('$id','en_US','$display_name');\n";

        if (!defined $default_value)
        {
            $sql .= "insert into habdbsync.p_array_device_config_meta ".
            "(id,model,type,config_key,config_type,is_mandtory) values".
            "('$id','$array_family', 'ARRAY','$config_key','$type','$is_mandatory');\n";
        }
        else
        {
            $sql .= "insert into habdbsync.p_array_device_config_meta ".
            "(id,model,type,config_key,config_type,default_value,is_mandtory) values".
            "('$id', '$array_family', 'ARRAY','$config_key','$type','$default_value','$is_mandatory');\n";
        }
    }

    $id = $array_family."_"."SDK_GENERATED";
    $sql .= "insert into habdbsync.p_array_device_config_meta ".
    "(id,model,type,config_key,config_type,default_value,is_mandtory) values".
    "('$id', '$array_family', 'ARRAY','SDK_GENERATED','META','1','1');\n";

    $id = $array_family."_"."PLUGIN_NAME";
    $default_value = $addon_id;
    $sql .= "insert into habdbsync.p_array_device_config_meta ".
    "(id,model,type,config_key,config_type,default_value,is_mandtory) values".
    "('$id', '$array_family', 'ARRAY','PLUGIN_NAME','META','$default_value','1');\n";


    $sql .= "insert into habdbsync.p_config_vendor_map ".
        "(vendor, model, method, discovery_method, type, info_text_id) ".
        " values".
        "('$vendor', '$model', '$array_family', 'Using $discv_method', 'ARRAY', '${array_family}_INFO_TEXT');\n";

    if ($sql ne "")
    {
        VRTS::Logger::log('error', "Updating DB with configuration meta information");
        $ret = exec_sql($sql);
        if ($ret)
        {
            return 1;
        }
    }
    if (add_resource_ids() != 0)
    {
        VRTS::Logger::log('error', "Failed add resource ids in the database");
        return 1;
    } 

    my @id_defs;

    @id_defs = keys %{$config_data->{ID_DEFS}} if (defined $config_data->{ID_DEFS});
    # If vdid formulae are defined then update the central vdid definitions file
    if (scalar(@id_defs) > 0)
    {
        VRTS::Logger::log ('debug', "Adding vdid definitions");
        my $vdid_def_data;

        my $si_conf_dir = File::Spec->catdir ($shared_dir, "si_conf_data");
        eval ( File::Path::mkpath($si_conf_dir, 0, 0755) );
        if($@)
        {
            $msg = "Cannot mkdir $@";
            VRTS::Logger::log ('error', $msg);
            print STDERR $msg;
            return 1;
        }

        # TODO lock the file si_vdid_def.conf to prevent any inconsistencies 
        # in case 2 plugins are getting setup at the same time
        my $buf;
        my $vdid_def_file = File::Spec->catfile ($si_conf_dir, "si_vdid_def.conf");
        if ( -f $vdid_def_file)
        {
            open (H, $vdid_def_file) or
               return (1, "failed to open file $vdid_def_file : $!");
            $buf = do { local $/; <H> };
            close (H);
    
            eval { $vdid_def_data = JSON::jsonToObj($buf)};
            if ($@)
            {
                $msg = "Invalid file $vdid_def_file\n";
                VRTS::Logger::log('error', $msg.":$@");
                return (1, $msg);
            }
        }

        foreach my $vendorid (keys %{$config_data->{ID_DEFS}})
        {
             foreach my $productid (keys %{$config_data->{ID_DEFS}->{$vendorid}})
             {
                 # Plugin vendor/product could be different from the SCSI vendor/product
                 # so store the plugin vendor/product in the definitions file to track
                 # which plugin installed these definitions

                 # A vendor/model could be discovered by multiple discovery methods.
                 # the last owner plugin to uninstall will remove the vdid definitions

                 my $plugin_def;
                 $plugin_def->{'plugin_vendor'} = $vendor;
                 $plugin_def->{'plugin_product'} = $model;
                 $plugin_def->{'plugin_family'} = $array_family;
                 $plugin_def->{'plugin_version'} = $version;

                 $vdid_def_data->{$vendorid}->{$productid}->{'plugin_info'}->{$array_family} = $plugin_def;

                 $vdid_def_data->{$vendorid}->{$productid}->{'scsi_data'} = 
                                      $config_data->{ID_DEFS}->{$vendorid}->{$productid}->{FIELDS};
                 $vdid_def_data->{$vendorid}->{$productid}->{'encl_vdid'} = 
                                      $config_data->{ID_DEFS}->{$vendorid}->{$productid}->{ENCL_VDID};
                 $vdid_def_data->{$vendorid}->{$productid}->{'lun_vdid'} =  
                                      $config_data->{ID_DEFS}->{$vendorid}->{$productid}->{LUN_VDID};
                 $vdid_def_data->{$vendorid}->{$productid}->{'page_list'} = 
                                      $config_data->{ID_DEFS}->{$vendorid}->{$productid}->{PAGE_LIST};
            }
        }
        eval { $buf = JSON::objToJson($vdid_def_data, {pretty => 1, indent => 1}); };
        if ($@)
        {
            $msg = "Failed to update vdid def data\n";
            VRTS::Logger::log('error', $msg.":$@");
            return (1, $msg);
        }
        open (H, ">$vdid_def_file") or
               return (1, "failed to open file $vdid_def_file : $!");
        print H $buf;
        close (H);
    }
    return 0;
}

# method executes the specified sql
sub exec_sql
{
    my $sql = shift;
    my $msg;
    my $db = "SFMdb3";
    my $rw_user  = "habdbsync";
    my $odbcini  =  "/etc/vx/VRTSsfmcs/.odbc.ini";
    $odbcini  =  File::Spec->catfile($appdircs,".odbc.ini") if ( $^O =~ /Win32/i);
    my $xdbadm = File::Spec->catfile($installdir, "bin", "xdbadm");
    my $db_conf = File::Spec->catdir($appdircs,"conf");

    if ($^O =~ /Win32/i)
    {
        $ENV{'SystemRoot'} =~ /([A-Za-z]:\\[A-Za-z0-9_ ]+)/;
        my $system32_dir = $1."\\system32".";".$1."\\system32\\wbem";
        $ENV{'PATH'} = $system32_dir;
    }
    else
    {
         $ENV{PATH}="/usr/bin:/bin:/sbin:/usr/sbin:/opt/VRTSsfmcs/asa11/bin";
    }

    my $ret = 0;
    my $data;
    my $sql_file = "";

    $sql_file = File::Temp::tempnam ($app_dir, "asdk");
    if ($sql_file ne "")
    {
       if (open (Q, ">$sql_file") == 0)
       {
           $msg = "Failed to creat sql file $sql_file : $@";
           VRTS::Logger::log ('error', $msg);
           print STDERR $msg;
           return 1;
       }
       print Q $sql;
       close (Q);
    }

    my $cmd = VRTS::Util::make_command($xdbadm,
                                       "-c",
                                       $db_conf,
                                       "-o",
                                       $odbcini,
                                       "-f",
                                       $sql_file,
                                       "-d",
                                       $db,
                                       "-u",
                                       $rw_user,"-v");
    $data = `$cmd 2>&1`;
    if ($?)
    {
        unlink ($sql);
        $msg = "update_table: sql $sql failed: $? :$data\n";
        VRTS::Logger::log ('error', $msg);
        print STDERR $msg;
        return 1;
    }
    unlink ($sql);
    return 0;
}

# method copies the plugin bits to target locations
sub copy_plugin_bits
{
    my $msg;
    my $ret = 0;

    VRTS::Logger::log ('debug', "Installing plugin bits");
    File::Path::rmtree($agentlet_dir);

    my @dirs = ($agentlet_dir, "$installdir/etc/SIPlugin", "$installdir/lib/modules/VRTS/Enclosure",
                "$installdir/lib/modules/VRTS/Enclosure/Discover");
    foreach my $dir (@dirs)
    {
        eval ( File::Path::mkpath($dir, 0, 0755) );
        if ($@)
        {
            $msg = "copy_plugin_bits: Failed to create directory $dir: $@\n";
            VRTS::Logger::log ('error', $msg);
            print STDERR $msg;
            return 1;
        }
    }
    
    if ($context eq "sdktest" && ($si_installed == 0))
    {
        # Copy SI Base components only if SI is not installed
        $plugin_bits{"SIBase/lib/modules/VRTS/Enclosure"} = {
            "dest" => "$installdir/lib/modules/VRTS/Enclosure",
            "remove" => 0
        };
        $plugin_bits{"SIBase/bin/array_driver.pl"} = {
            "dest" => "$installdir/bin/array_driver.pl",
            "remove" => 0
        };
        $plugin_bits{"SIBase/operations"} = {
            "dest" =>"$installdir/operations",
            "remove" => 0
        };
        $plugin_bits{"Constants"} = {
            "dest" => "$installdir/agentlet/VRTS/Constants",
            "remove" => 0
        };
    }
    chdir $addon_dir;
    foreach my $file (keys %plugin_bits)
    {
        if (-f $file)
        {
            VRTS::Logger::log ('debug', "Installing $file at $plugin_bits{$file}->{dest}");
            unlink ($plugin_bits{$file}->{dest});
            $ret = File::Copy::copy ($file, $plugin_bits{$file}->{dest});
            if ($ret == 0)
            {
                $msg = "copy_plugin_bits: Failed to copy file $file: $!\n";
                VRTS::Logger::log ('error', $msg);
                print STDERR $msg;
                return 1;
            }
        }
        elsif (-d $file)
        {
            my $dest = $plugin_bits{$file}->{dest};

            if ($plugin_bits{$file}->{remove})
            {
                File::Path::rmtree($plugin_bits{$file}->{dest});
            }
            else
            {
                $dest =~ m/(.+)(\/[^\/]+)/;
                $dest = $1 if ($^O !~ /Win32/i);
            }

            VRTS::Logger::log ('debug', "Installing $file at $dest");
            eval { VRTS::Util::copy_recursively($file, $dest)};
            if ($@)
            {
                $msg = "copy_plugin_bits: failed to copy directory $file";
                VRTS::Logger::log ('error', $msg);
                print STDERR $msg;
                return 1;
            }
        }
    }
    return 0;
}

# method registers the family with the domain.
# this enables the reporting framework to send data to the configured CMS
sub register_family_with_domains
{
    my $msg;
    my $fn = "register_family_with_domains";

    my $cmd1  = File::Spec->catfile($installdir,"bin","mh_ctl.pl");
    my $cmd = make_command($perl, $cmd1, "--family", $array_family, "--register", "--domain", "sfm://", "--schedule", "1440");
    my $ret = `$cmd`;
    if ($ret)
    {
        $msg = "$fn: failed to register family with domains";
        VRTS::Logger::log ('error', $msg);
        return 1;
    }
    $cmd = make_command($perl, $cmd1, "--family", $array_family, "--pause");
    `$cmd`;

    return 0;
}

# Method cleans up plugin components
sub cleanup
{
    my $fn = "cleanup";
    my $ret = 0;
    my $msg;

    # Cleanup plugin related bits/directories
    foreach my $file (@cleanup_files)
    {
        if (-d $file)
        {
            VRTS::Logger::log('debug', "Removing directory $file");
            File::Path::rmtree($file);
        }
        elsif (-f $file)
        {
            VRTS::Logger::log('debug', "Removing file $file");
            unlink($file);
        }
    }

    return 0 if ($context eq "sdktest");

    # unregister the plugin family
    VRTS::Logger::log ('debug', "Unregistering the family $array_family");

    my $cmd  = File::Spec->catfile($installdir, "bin", "mh_ctl.pl");
    $cmd = make_command($perl, $cmd, "--family", $array_family, "--unregister", "--domain", "sfm://");
    `$cmd`;
    if ($?)
    {
       VRTS::Logger::log ('error', "$fn:Failed to unregister $array_family family");
    }

    # Get list of devices configured for this family and de-schedule them
    my $schedules = VRTS::Schedule::read_sched();
    my $vdid;
    my $sql = "";

    VRTS::Logger::log ('debug', "De-scheduling discovery for $array_family devices");
    foreach my $f (@{$schedules})
    {
        if ($f->{command} =~ /array_driver.* --family $array_family --hidden --context vardir=(.+)/)
        {
            $vdid = $1;
            VRTS::Logger::log ('debug', "De-schedule $f->{id}:$vdid");
            VRTS::Schedule::del_sched($f->{id});
            $sql .= "delete from habdbsync.p_array_subsystem where id='$vdid';\n" if ($active_cms);
        }
    }
    my $dummy_mhdriver_sched_id = "discovery_".$array_family;
    VRTS::Schedule::del_sched($dummy_mhdriver_sched_id);

    if ($active_cms)
    {
        my $inc_agentlet = File::Spec->catfile($installdir, "agentlet");
        push @INC, $inc_agentlet;
        
        if(eval "require VRTS::Enclosure::Utils")
        { 
            if (defined $vdid)
            {
                VRTS::Logger::log ('debug', "Fetch device configurations for $vdid");
                # Delete all device configurations for this array family
                my $array_config = &{\&{"VRTS::Enclosure::Utils::get_config_by_vendor_id"}}($vdid);
                if (defined $array_config && $array_config->[0]->{METHOD} eq  $array_family)
                {
                    VRTS::Logger::log ('debug', "Found device configurations.config_id $array_config->[0]->{ID}");
                    $sql .= "delete from HABDBSYNC.P_ARRAY_CONFIG where id='$array_config->[0]->{ID}';";
                }
            }
        }
        else
        {
            VRTS::Logger::log('error', "$fn. Failed to load module : $@");
        }

        if (defined $config_data)
        {
            foreach my $key (keys %{$config_data->{'DEVICE_CONFIG_META'}})
            { 
                my $id = "ARRAY_".$array_family."_".$key;
                $sql .= "delete from habdbsync.p_ui_resource where id= '$id';".
                        "delete from habdbsync.p_ui_res_bundle where id='$id';".
                        "delete from habdbsync.p_array_device_config_meta where id='$id';";
                VRTS::Logger::log ('debug', "removing meta for $id");
            }
            $sql .= "delete from habdbsync.p_config_vendor_map where vendor='$vendor' and ".
                     "model='$model' and method='$array_family' and type='ARRAY';\n";
            VRTS::Logger::log ('debug', "Removing device configuration data from Database");
            if ($sql ne "")
            {
                $ret = 1 if (exec_sql($sql));
            }
        }


        # Remove vdid definitions for this array vendor-model
        # TODO lock the file si_vdid_def.conf to prevent any inconsistencies
        # in case 2 plugins are getting setup/unsetup at the same time
        my $vdid_def_file = File::Spec->catfile ($shared_dir, "si_conf_data","si_vdid_def.conf");
        if ( -f $vdid_def_file && defined $vendor && defined $model)
        {
            VRTS::Logger::log ('debug', "Check if vdid definitions have to be removed");
            open (H, $vdid_def_file) or
               return (1, "failed to open file $vdid_def_file : $!");
            my $buf = do { local $/; <H> };
            close (H);

            my $vdid_def_data;
            eval { $vdid_def_data = JSON::jsonToObj($buf)};
            if ($@)
            {
                $msg = "Invalid file $vdid_def_file\n";
                VRTS::Logger::log('error', $msg.":$@");
            }
            foreach my $vendorid (keys %$vdid_def_data)
            {
                foreach my $productid (keys %{$vdid_def_data->{$vendorid}})
                {
                    if (exists $vdid_def_data->{$vendorid}->{$productid}->{plugin_info}->{$array_family})
                    {
                        my @plugin_owners = keys %{$vdid_def_data->{$vendorid}->{$productid}->{plugin_info}};
                        if (scalar(@plugin_owners) == 1)
                        { 
                            # Last owner for the vdid definitions. Safe to remove the definition.
                            VRTS::Logger::log('debug', "Remove vdid definitions for $vendorid($productid)");
                            delete $vdid_def_data->{$vendorid}->{$productid};
                        }
                        else
                        {
                            # Muliple owners for the definition so just unregister this plugin as the owner
                            # and do not remove the definition.
                            VRTS::Logger::log('debug',"Un-Registered family $array_family as owner for ".
                               "vdid definition $vendorid($productid)"); 
                            delete $vdid_def_data->{$vendorid}->{$productid}->{plugin_info}->{$array_family};
                        }
                    }
                }
                my @models = keys %{$vdid_def_data->{$vendorid}};
                if (scalar(@models) == 0)
                {
                    # No products exists for this vendor. 
                    VRTS::Logger::log ('debug', "Removing all vdid definitions for Vendorid $vendorid");
                    delete $vdid_def_data->{$vendorid};
                }
            }
 
            $buf = JSON::objToJson($vdid_def_data, {pretty => 1, indent => 1});

            open (H, ">$vdid_def_file") or
               return (1, "failed to open file $vdid_def_file : $!");
            print H $buf;
            close (H);
        }
    }
    return $ret;
}

sub get_env
{
    my $ret = 0;
    my $msg;

    VRTS::Logger::log ('debug', "addon-dir is $addon_dir");

    $si_installed = 0;
    my $info = new VRTS::AddOnInfo;
    my $ret = $info->initialize();
    if($ret == 0)
    {
        my %f_info = %{$info->get_features()};
        for my $feature (keys %f_info)
        {
             if ($f_info{$feature}{'ID'} =~ /^VRTSsfmsi-/)
             {
                 $si_installed = 1;
                 $si_ver = $f_info{$feature}{'VERSION'};
                 VRTS::Logger::log('debug', "SI Add-on version $si_ver installed");
                 last;
             }
         }
    }

    if ($context ne "sdktest")
    {
        my $cms_info = VRTS::Util::get_cms_config_info();
        if ($cms_info->{'cs'} == 0)
        {
            $msg = "Current host is not a Management Server.\n".
                   "SI Plug-in should be installed only on a Management Server\n";
            print STDERR $msg;
            VRTS::Logger::log ('error', $msg);
            return 1;
        }

        $ha = $cms_info->{'ha'};
        if ($ha)
        {
            if ($cms_info->{'active_node'} == 1)
            {
               # this is a CMS HA active node
               $active_cms = 1;
            }
        }
        else
        {
            $active_cms = 1;
        }
        VRTS::Logger::log('debug', "active_cms = $active_cms. ha = $ha");
    }

    # TODO how do we protect the case where same array/family are discovered via a different
    # mechanism? 
    @cleanup_files = ();

    my $config_file = File::Spec->catfile ($addon_dir, "plugin_def.conf");
    if (open (H, $config_file) == 0)
    {
        $msg = "failed to open file $config_file : $@";
        VRTS::Logger::log ('error', $msg);
    }
    else
    {
        my $buf = do { local $/; <H> };
        close (H);
        $config_data;
        eval { $config_data = JSON::jsonToObj($buf)};
        if ($@)
        {
            $msg = "Invalid config file $config_file : $@\n";
            VRTS::Logger::log ('error', $msg);
            return 1;
        }
        else
        {
           $vendor = $config_data->{VENDOR};
           $model = $config_data->{MODEL}; 
           $array_family = $config_data->{ARRAY_PLUGIN_FAMILY};
           $version = $config_data->{ARRAY_PLUGIN_VERSION};
           $discv_method = $config_data->{ARRAY_PLUGIN_METHOD};
           $addon_id = $config_data->{PLUGIN_ADDON_ID};

           if (!defined $vendor || $vendor eq "" 
               || !defined $model || $model eq ""
               || !defined $array_family || $array_family eq ""
               || !defined $version || $version eq "")
           {
               VRTS::Logger::log('error', "Invalid plugin config file. Missing vital information");
               return 1;
           }
           $version =~ s/^v//;

           VRTS::Logger::log('debug', "Vendor [$vendor].Model [$model].".
                   "Family [$array_family].Plugin vesion [$version].Addonid $addon_id");
        }
    }

    my $array_app_dir = File::Spec->catdir ($app_dir, $array_family);
    $agentlet_dir = File::Spec->catdir ($installdir, "agentlet", "VRTS", $array_family);

    push (@cleanup_files, $agentlet_dir);
    push (@cleanup_files, $array_app_dir);

    $plugin_bits{"S10discover.pm"} = {
        "dest" => "$agentlet_dir/S10discover.pm",
        "remove" => 1
    };
    $plugin_bits{"tdf"} = {
        "dest" => "$installdir/etc/tdf/$array_family",
         "remove" => 1
    };
    $plugin_bits{"${array_family}.pm"} = {
        "dest" => "$installdir/lib/modules/VRTS/Enclosure/Discover/${array_family}.pm",
        "remove" => 1
    };
    $plugin_bits{"SIPlugin"} = {
        "dest" => "$installdir/lib/modules/VRTS/Enclosure/SIPlugin",
        "remove" => 0
    };
    $plugin_bits{"cliobjects.def"} = {
        "dest" => "$installdir/etc/SIPlugin/cliobjects.def",
        "remove" => 0
    };
    $plugin_bits{"$array_family"} = {
        "dest" => "$installdir/etc/SIPlugin/$array_family",
        "remove" => 1
    };


    foreach my $file (keys %plugin_bits)
    {
        push (@cleanup_files, $plugin_bits{$file}->{dest}) if ($plugin_bits{$file}->{remove});
    }
    return 0;
}

sub add_resource_ids
{
    my $sql;
    my $res_hash;
    my $ret = 0;

    if(eval "require VRTS::Enclosure::SIPlugin::Util")
    {
        $res_hash = &{\&{"VRTS::Enclosure::SIPlugin::Util::get_error_res_map"}}();
        if (defined $res_hash)
        {
            foreach my $id (keys %$res_hash)
            {
                my $en_msg = &{\&{"VRTS::Enclosure::SIPlugin::Util::get_error"}}(undef, $id);
                my $res_id = $res_hash->{$id};
                $sql .= "INSERT INTO HABDBSYNC.UI_VIEW_RESOURCE_R VALUES ('$res_id');\n".
                "INSERT INTO HABDBSYNC.UI_VIEW_ENUM_DEF_R (OBJECT, PROPERTY, VALUE, INT_VALUE, RESOURCE_ID)".
                " VALUES ('ARRAY','PROBE','$res_id', null,'$res_id');\n".
                "INSERT INTO HABDBSYNC.UI_VIEW_RES_BUNDLE_R VALUES ".
                "('$res_id', 'en_US', '$en_msg');\n";
            }
        }
    }
    if (defined $sql && $sql ne "")
    {
        $ret = 1 if (exec_sql($sql));
    }
    return $ret;
}
