#!/opt/VRTSsfmh/bin/perl
#$Id$
#$Copyright$

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }
use strict;
use Getopt::Long;
use JSON;
use VRTS::Paths;
use File::Spec;
use File::Path;
use File::Copy;
use File::Temp;
use VRTS::Util;
use Archive::Tar;
use CGI;
use VRTS::SISdk::Fabricate;
use VRTS::SISdk::Util;

use VRTS::Ini;
use IO;
use Inline::Files;
my $fh = new IO::Handle;

my $appdir = VRTS::Paths::get_path("AppDir");
my $log_dir = File::Spec->catdir($appdir,"logs");
my $installdir  = VRTS::Paths::get_path ("InstallDir");

#---------------------------------------
main();

sub main{
    my $query = new CGI();
    my $log = new VRTS::Logger(Module => 'sisdkui', Logdir => $log_dir, Level => 'debug', Logfile => 'sisdkui.log', UmiString => '');

    print "Content-type: text/html\n\n";    
    my $readOnly  = $query->param('readOnly');

    if ($readOnly) { #show on the test result screen
       print testResultScreenTestParameters($query);
    }else{
       print configTestParameters($query);
    }
}


sub configTestParameters{
    my $query = shift;
    my $paramDisplay ="";
    my $commandScript =  $query->cookie('commandScript');
    my $readOnly  = $query->param('readOnly');
    my $configParamSets = $query->cookie('iConfigureParams');	
 
    my $pluginName = $query->cookie('iName');

    if (!(defined $configParamSets) || !$configParamSets  ){
        my $javaScript = '<SCRIPT id="testRunScript" type="text/javascript"> function validateTestInput(){ return true; }</SCRIPT>';
        return $javaScript; 
    }
    
    my $paramDisplay = join "", $paramDisplay,  '<table align="center" border="0">';              
    $paramDisplay = join "", $paramDisplay, "<tr>";  	  
    $paramDisplay = join "", $paramDisplay, ' <td class="line" colspan="2"><strong> Test configuration parameters for plug-in [', $pluginName, ']</strong></td>';	
    $paramDisplay = join "", $paramDisplay, "</tr>";
    $paramDisplay = join "", $paramDisplay, "<tr>";  	  
    $paramDisplay = join "", $paramDisplay, ' <td colspan="2" hight="1">&nbsp;</td>';	
    $paramDisplay = join "", $paramDisplay, "</tr>";
    $paramDisplay = join "", $paramDisplay, '<tr bgcolor="#bbd8e5">';  	  
    $paramDisplay = join "", $paramDisplay, '<td class="contentpane1">&nbsp;Parameter</td>';
    $paramDisplay = join "", $paramDisplay, '<td class="contentpane1">&nbsp;Value <input type="hidden" name="commandScript" id="commandScript" value="$commandScript"></td>';
    $paramDisplay = join "", $paramDisplay, "</tr>";
                   
    my @configParamArray = split(':::', $configParamSets);
    my $size = @configParamArray;
    my $configParmCount = 0;
    my $javaScript = '<SCRIPT id="testRunScript" type="text/javascript"> function validateTestInput(){ var message = ""; ';
    foreach my $oneSet (@configParamArray) {
        if (length($oneSet)>3) {
            my @params = split(',,,', $oneSet);
            my $name = @params[0];
            my $cliName = @params[1];
            my $defaultValue = @params[2];
            my $datatype = @params[3];
           # my $is_mandatory = @params[4];
            my $is_mandatory = @params[4];
            if ($is_mandatory eq 0) {
                $is_mandatory = "No";
            }else{
                $is_mandatory = "Yes";
            }
            my $testValue =  @params[5];

            my $nameField = join "", "nameField_",  $configParmCount;
            my $cliNameField = join "", "cliNameField_",  $configParmCount;
            my $valueField = join "", "defaultValueField_",  $configParmCount;
            my $testValueField = join "", "testValueField_",  $configParmCount;
            my $dataTypeField = join "", "dataTypeSelect_",  $configParmCount;
            my $mandatoryField = join "", "MandatorySelect_",  $configParmCount;
            my $configParmEntry = "";

            #if ($configParmCount % 2 == 1) {
            #    $configParmEntry = '<tr bgcolor="#fff">';
            #}else{
            $configParmEntry = '<tr bgcolor="#eee">';
            #}

            $configParmEntry = join "", $configParmEntry, "<td><input name='$nameField'  type='text' class='inputbox' id='$nameField' value='$name' readonly='readonly'/></td>	";
            if ($readOnly) {
                if ("PASSWORD_STRING" eq $datatype) {
                   $configParmEntry = join "", $configParmEntry, "<td><input name='$testValueField' type='password' class='inputbox' id='$testValueField' value='$testValue' readonly='readonly'/>";
                }else{
                   $configParmEntry = join "", $configParmEntry, "<td><input name='$testValueField' type='text' class='inputbox' id='$testValueField' value='$testValue' readonly='readonly'/>";
                }
            }else{
                if ("PASSWORD_STRING" eq $datatype) {
                   $configParmEntry = join "", $configParmEntry, "<td><input name='$testValueField' type='password' class='inputbox' id='$testValueField' value='$defaultValue'/>";
                }else{
                   $configParmEntry = join "", $configParmEntry, "<td><input name='$testValueField' type='text' class='inputbox' id='$testValueField' value='$defaultValue'/>";
                }
               
                if ($is_mandatory eq "Yes") {
                   $javaScript .= ' if (isEmpty(document.getElementById("' . $testValueField . '").value)) { ';
                   $javaScript .= '  message = message + "' . $name . '" + " is required.\n";  ';
                   $javaScript .= ' } ';
                }
            }
            $configParmEntry = join "", $configParmEntry, "<input type='hidden' name='$cliNameField'  id='$cliNameField' value='$cliName'/>";  
            $configParmEntry = join "", $configParmEntry, "<input type='hidden' name='$valueField' id='$valueField' value='$defaultValue'>";
            $configParmEntry = join "", $configParmEntry, "<input type='hidden' name='$dataTypeField' id='$dataTypeField' value='$datatype'>";
            $configParmEntry = join "", $configParmEntry, "<input type='hidden' name='$mandatoryField' id='$mandatoryField' value='$is_mandatory'>";
            $configParmEntry = join "", $configParmEntry, "</td></tr>";
            $paramDisplay = join "", $paramDisplay, $configParmEntry;
            $configParmCount = $configParmCount + 1;
        }
    }
    $javaScript .= ' if (message==""){ return true} else{alert(message); }}     </SCRIPT>';

    $paramDisplay = join "", $paramDisplay, "</table>",  $javaScript;
    return $paramDisplay;
}

sub testResultScreenTestParameters{
    my $query = shift;
    my $testCommandFile = $query->cookie('testCommandFile');	
    my $testOption = $query->cookie('testOption');
    my $pluginName = $query->cookie('iName');
    my $paramDisplay = "";
    VRTS::Logger::log('debug',"testOption=$testOption");
    my $testFileName = VRTS::SISdk::Util::GetBasename($testCommandFile);  

    if ("1" eq  $testOption) {
        $paramDisplay = join "", $paramDisplay,  '<table align="center" border="0">';          
        $paramDisplay = join "", $paramDisplay, "<tr>";  	  
        $paramDisplay = join "", $paramDisplay, ' <td class="line" colspan="2" align="center"><strong> Test plug-in [', $pluginName, '] with collected data</strong></td>';	
        $paramDisplay = join "", $paramDisplay, "</tr>";
        $paramDisplay = join "", $paramDisplay, '<tr>';  	  
        $paramDisplay = join "", $paramDisplay, "<tr>";  	  
        $paramDisplay = join "", $paramDisplay, ' <td colspan="2" hight="1">&nbsp;</td>';	
        $paramDisplay = join "", $paramDisplay, "</tr>";        
        $paramDisplay = join "", $paramDisplay, '<td  colspan="2" align="center"> Test file name: ', $testFileName, '</td>';
        $paramDisplay = join "", $paramDisplay, "</tr>";
        $paramDisplay = join "", $paramDisplay, "<tr>";  	  
        $paramDisplay = join "", $paramDisplay, ' <td colspan="2" hight="1">&nbsp;</td>';	
        $paramDisplay = join "", $paramDisplay, "</tr>";
        $paramDisplay = join "", $paramDisplay, "</table>";
        return $paramDisplay;
    }else{
        my $configParamSets = $query->cookie('iConfigureParams');	
        if (!(defined $configParamSets) || !$configParamSets  ){
            $paramDisplay = join "", $paramDisplay,  '<table align="center" border="0">';          
            $paramDisplay = join "", $paramDisplay, "<tr>";  	  
            $paramDisplay = join "", $paramDisplay, ' <td class="line" colspan="2" align="center"><strong> Test plug-in ', $pluginName, ' with live data</strong></td>';	
            $paramDisplay = join "", $paramDisplay, "</tr>";
            $paramDisplay = join "", $paramDisplay, '<tr>';  	  
            $paramDisplay = join "", $paramDisplay, "<tr>";  	  
            $paramDisplay = join "", $paramDisplay, ' <td colspan="2" hight="1">&nbsp;</td>';	
            $paramDisplay = join "", $paramDisplay, "</tr>";        
            $paramDisplay = join "", $paramDisplay, '<td  colspan="2" align="center"> No Parameter Entered</td>';
            $paramDisplay = join "", $paramDisplay, "</tr>";
            $paramDisplay = join "", $paramDisplay, "<tr>";  	  
            $paramDisplay = join "", $paramDisplay, ' <td colspan="2" hight="1">&nbsp;</td>';	
            $paramDisplay = join "", $paramDisplay, "</tr>";
            $paramDisplay = join "", $paramDisplay, "</table>";
            return $paramDisplay;
         }else{
           return configTestParameters($query);
        }
    }

    
}
