#!/opt/VRTSsfmh/bin/perl
#$Id$
#$Copyright$
BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }

use strict;
use Getopt::Long;
use JSON;
use VRTS::Paths;
use File::Spec;
use File::Path;
use File::Copy;
use File::Temp;
use VRTS::Util;
use Archive::Tar;
use CGI;
use VRTS::SISdk::Fabricate;
use VRTS::SISdk::Util;

use VRTS::Ini;
use IO;
use Inline::Files;
my $fh = new IO::Handle;

my $appdir = VRTS::Paths::get_path("AppDir");
my $log_dir = File::Spec->catdir($appdir,"logs");
my $installdir  = VRTS::Paths::get_path ("InstallDir");

my $templateConfigFile = File::Spec->catfile($installdir,"etc", "SISdk", "params.conf");

main();

sub main{
    my $query = new CGI();
    my $ret = 0;
    my $addOnAction = $query->param("addOnAction");
    my $log = new VRTS::Logger(Module => 'sisdkui', Logdir => $log_dir, Level => 'debug', Logfile => 'sisdkui.log', UmiString => '');

    VRTS::Logger::log('debug', "<addOnAction>". $addOnAction);

    if ($addOnAction eq "uploadForEdit" || $addOnAction eq "uploadForTest") {

       my ($re, $plugin_conf)=getAddOnConfig($query,  $appdir, $installdir);
       if ($re) {
              my $baseInputSfaFileName = $query->param("iSfaFile");
              my $script ="";
               $script .= "<script>"; 
               $script .= "parent.alertMessage('Failed to process file $baseInputSfaFileName<p> Please check log for details', ''); "; 
               $script .= "</script>";
               print "Content-type: text/html\n\n"; 
               VRTS::Logger::log('debug', " return script $script:$@");
               print $script;
       }
       VRTS::Logger::log('debug', "<plugin_conf for upload for edit>". $plugin_conf);
       if ($plugin_conf) {
           open (H, $plugin_conf) or             
            return (1, "failed to open file $plugin_conf");
            my $templateConfigbuf = do { local $/; <H> };
            close (H);   

            my $conf_json;
            eval { $conf_json = JSON::jsonToObj($templateConfigbuf)};
            if ( $@)
            {
                my $msg = "Invalid configuration file\n";
                VRTS::Logger::log('error', $msg.":$@");
                printErrorMessage();
                exit(1);
            }
            my $hasCliParam = jsonConfigToCookie($conf_json, $query);
            if ($addOnAction eq "uploadForTest") {
               VRTS::Logger::log('debug', "upload plug-in addon to test.:$@");
               my $url = "../htdocs/sisdk_test.html";
               if (!$hasCliParam) {
                   $url = "../cgi-bin/sisdk_ui_manageAddOnConfiguration.pl?runTest=yes&commandScript=";
               }
               VRTS::Logger::log('debug', "url is  $url:$@");
               my $inputSfaFileName = $query->param("iSfaFile");

               my $script ="";
               $script .= "<script>"; 
               $script .= "parent.alertMessage('$inputSfaFileName is uploaded successfully.<p> Click &quot;Test&quot; button to launch test.', '$url'); "; 
               $script .= "</script>";
               print "Content-type: text/html\n\n"; 
               VRTS::Logger::log('debug', " return script $script:$@");
               print $script;
            }else{
               my $inputSfaFileName = $query->param("iSfaFile");
               my $url = "../htdocs/sisdk_uploadScript.html";
               my $script ="";
               $script .= "<script>"; 
               $script .= "parent.alertMessage('$inputSfaFileName is uploaded successfully.<p> Click &quot;Edit&quot; button to edit the plug-in addon.', '$url'); "; 
               $script .= "</script>";
               print "Content-type: text/html\n\n"; 
               VRTS::Logger::log('debug', " return script $script:$@"); 
               print $script;
            }
        }else{
            printErrorMessage();
            exit(0);            
        }   
    }else{
        my $OriginalConfig;

        my $conf_file = File::Temp::tempnam ($appdir, "pluginconf");
        my $conf_fileCookie= $query->cookie(-name=>"conf_file", -value=>"$conf_file");	
        print "Set-Cookie:$conf_fileCookie;\n"; 

        File::Copy::copy($templateConfigFile, $conf_file);
        
        ($ret, $OriginalConfig) = getJsonDataFromTemplateConfig($conf_file);

        if ($ret) {
            print "Location: ../htdocs/sisdk_error.html";
        }
        my $updatedConfig = updateConfiData($OriginalConfig, $query);
        ($ret, $updatedConfig) = writeNewConfigToTmpFile($updatedConfig, $conf_file);
        
        if ($ret) {
            print "Location: ../htdocs/sisdk_error.html";
        }

        my $configParamSets = $query->cookie('iConfigureParams');	

        my $test =  $query->param("runTest");
        my $baseConfFileName   = "\$VARDIR/" . VRTS::SISdk::Util::GetBasename($conf_file);
        
        if ("yes" eq $test) {
            my $testCommandFile = $query->param("testCommandFile");
            my $testOption = $query->param("testOption");                     
            my $config = JSON::jsonToObj($updatedConfig);
            my $testLogFileName = getTestLogFileName($config->{ARRAY_PLUGIN_METHOD}, $config->{VENDOR}, $config->{MODEL});
            my $baseTestLogFileName= "\$VARDIR/" . VRTS::SISdk::Util::GetBasename($testLogFileName);
            my $baseTestCommandFile=  $testCommandFile;
            if ($testOption) {
                print "Location: ../htdocs/sisdk_testResult.html?config-data=$baseConfFileName&test=1&test-source=$testOption&test-source-file=$baseTestCommandFile&test-log=$baseTestLogFileName\n\n";
            }else{
                print "Location: ../htdocs/sisdk_testResult.html?config-data=$baseConfFileName&test=1&test-source=$testOption&test-log=$baseTestLogFileName\n\n";
            }                
            
        } else {
            print "Location: ../htdocs/sisdk_addon.html?config-data=$baseConfFileName\n\n";
        }

    }
}
sub printErrorMessage{
    my $script ="";
       $script .= "<script>"; 
       $script .= "parent.alertMessage('Failed to process file <p> Please check log for details', ''); "; 
       $script .= "</script>";
       print "Content-type: text/html\n\n"; 
       VRTS::Logger::log('debug', " return script $script:$@");
       print $script;
}
sub getTestLogFileName{
    my $discv_method = shift;
    my $vendor = shift;
    my $model = shift;
    my $logFileName =  $discv_method;
    if (defined $vendor && defined $model)
    {
       $logFileName = ${vendor}."_".${model}."_".${logFileName}."_".$$.".log";
    }else{
       $logFileName = ${logFileName}."_".$$.".log";
    }     
    my $filePathName = File::Spec->catfile ($log_dir, "$logFileName");
    return $filePathName;
}
sub getJsonDataFromTemplateConfig{
    my $tmpConfigFile =  $_[0];
    open (H, $tmpConfigFile) or 
      return (1, "failed to open file $tmpConfigFile");
    my $templateConfigbuf = do { local $/; <H> };
    close (H);   

    my $templateConfig;	
    eval { $templateConfig = JSON::jsonToObj($templateConfigbuf)};
    if ( $@)
    {
        my $msg = "Invalid configuration file\n";
        VRTS::Logger::log('error', $msg.":$@");
        return (1, $msg);
    }
    return (0, $templateConfig);
}

sub writeNewConfigToTmpFile{
    my($configureData, $conf_file) = @_;

    my $config_Json_data = JSON::objToJson($configureData, {pretty => 1, indent => 1});
    if (open (H, ">$conf_file") == 0)
    {
        my $msg = "Failed to create plug-in config script\n";
        VRTS::Logger::log('error', $msg);
        return 1, $msg;
    }
    print H $config_Json_data;
    close(H);
    return 0, $config_Json_data;
}

sub updateConfiData{  
    my ($OriginalConfig, $query) = @_;
    $OriginalConfig->{"ARRAY_PLUGIN_NAME"} = $query->cookie('iName');
    $OriginalConfig->{"ARRAY_PLUGIN_VERSION"} = $query->cookie('iVersion');
    $OriginalConfig->{"VENDOR"} = $query->cookie('iVendor');
    $OriginalConfig->{"MODEL"} = $query->cookie('iModel');
    
    $OriginalConfig->{"ARRAY_PLUGIN_METHOD"} = $query->cookie('iFamily');
    $OriginalConfig->{"DISCOVERY_INFO"}->{"ENCL_INFO"} = $query->cookie('cliCommand');

    $OriginalConfig->{"CAPABILITIES"}->{"PDEV"} = 0;
    $OriginalConfig->{"OBJECT_NAMES"}->{"PDEV"} = "";
    if ("DiscoveryPhysicalDisk" eq $query->cookie('chDiscoveryPhysicalDisk')) {      
       $OriginalConfig->{"CAPABILITIES"}->{"PDEV"} = 1;
       if ( defined $query->cookie('iPhysicalDiskTerm')) {
          $OriginalConfig->{"OBJECT_NAMES"}->{"PDEV"} = $query->cookie('iPhysicalDiskTerm');
       }
    }
    
    $OriginalConfig->{"CAPABILITIES"}->{"ARRAY_PORT"} = 0;
    if ("DiscoverArrayPort" eq $query->cookie('chDiscoverArrayPort')) {	   
      $OriginalConfig->{"CAPABILITIES"}->{"ARRAY_PORT"} = 1;		
    }

    $OriginalConfig->{"CAPABILITIES"}->{"ADAPTER"} = 0;
    $OriginalConfig->{"CAPABILITIES"}->{"RG_PDEV_MAP"} = 0;
    $OriginalConfig->{"CAPABILITIES"}->{"RG_LDEV_MAP"} = 0;

    if ("DiscoverAdapter" eq $query->cookie('chDiscoverAdapter')) {	
         $OriginalConfig->{"CAPABILITIES"}->{"ADAPTER"} = 1;		
    }

    $OriginalConfig->{"CAPABILITIES"}->{"RG"} = 0;
    if ("DiscoverRaidGroup" eq $query->cookie('chDiscoverRaidGroup')) {
       $OriginalConfig->{"CAPABILITIES"}->{"RG"} = 1;
     
       if ( defined $query->cookie('iRaidGroupTerm')) {
          $OriginalConfig->{"OBJECT_NAMES"}->{"RAIDGROUP"} = $query->cookie('iRaidGroupTerm');
       }
       if ( "RaidGroupPhysicalMapping" eq $query->cookie('chRaidGroupPhysicalMapping')) {
          $OriginalConfig->{"CAPABILITIES"}->{"RG_PDEV_MAP"} = 1;
       }
       if ( "RaidGroupLogicalDiskMapping" eq $query->cookie('chRaidGroupLogicalDiskMapping')) {
          $OriginalConfig->{"CAPABILITIES"}->{"RG_LDEV_MAP"} = 1;
       }
    }
     
    $OriginalConfig->{"DISCOVERY_INFO"}->{"COMMAND_LOCATION"} = $query->cookie('commandScript');	 
    $OriginalConfig->{"DISCOVERY_INFO"}->{"DISCOVERY_CMD"} = $query->cookie('iDiscoveryFileName');	 
    
    my $iVendorId = $query->cookie('iVendorId'); 
    my $iProductId = $query->cookie('iProductId'); 
    
    my $deviceIdentifer = $query->cookie('deviceIdentifer');
    if ($deviceIdentifer && $iVendorId && $iProductId) {    
        $OriginalConfig->{"ID_DEFS"}->{$iVendorId}->{$iProductId}->{"FIELDS"}= JSON::jsonToObj($deviceIdentifer);

        my $encIdentiferFormula = $query->cookie('EncIdentiferFormula');
        $encIdentiferFormula  =~s/\$\{VENDOR\}/\$\{V\}/g;
        $encIdentiferFormula  =~s/\$\{PRODUCT\}/\$\{P\}/g;
        $encIdentiferFormula  =~s/\$\{Field([\d]+)\}/\$\{F$1}/g;
        if ($encIdentiferFormula) {
           $OriginalConfig->{"ID_DEFS"}->{$iVendorId}->{$iProductId}->{"ENCL_VDID"}=$encIdentiferFormula;
        }else{
           $OriginalConfig->{"ID_DEFS"}->{$iVendorId}->{$iProductId}->{"ENCL_VDID"}="";
        }
    }
    

    $OriginalConfig->{"CAPABILITIES"}->{"LDEV"} = 0;
    $OriginalConfig->{"OBJECT_NAMES"}->{"LDEV"} = ""; 
    if ("DiscoverLogicalDevices" eq $query->cookie('chDiscoverLogicalDisk')) {
       my $iLogicalDiskTerm = $query->cookie('iLogicalDiskTerm');
       $OriginalConfig->{"CAPABILITIES"}->{"LDEV"} = 1;
       $OriginalConfig->{"OBJECT_NAMES"}->{"LDEV"} = $iLogicalDiskTerm;
  
       $OriginalConfig->{"CAPABILITIES"}->{"HOST_LUN_MAPPING"} = 0;
       if ("DiscoverLogHostMapping" eq $query->cookie('chDiscoverLogHostMapping')) {
          $OriginalConfig->{"CAPABILITIES"}->{"HOST_LUN_MAPPING"} = 1;
       }

       $OriginalConfig->{"CAPABILITIES"}->{"LDEV_PDEV_MAP"} = 0;
       if ("DiscoverLogPhyMapping" eq $query->cookie('chDiscoverLogPhyMapping')) {
          $OriginalConfig->{"CAPABILITIES"}->{"LDEV_PDEV_MAP"} = 1;
       }

       $OriginalConfig->{"OBJECT_NAMES"}->{"META_LDEV"} = "";
       if ("MetaLogicalDisk" eq $query->cookie('chMetaLogicalDisk')) {
          my $iTermMetaLogicalDisk = $query->cookie('iTermMetaLogicalDisk');          
          $OriginalConfig->{"OBJECT_NAMES"}->{"META_LDEV"} = $iTermMetaLogicalDisk;
       }

       my $logicalIdentiferFormula = $query->cookie('LogicalIdentiferFormula');   
          $logicalIdentiferFormula  =~s/\$\{VENDOR\}/\$\{V\}/g;
          $logicalIdentiferFormula  =~s/\$\{PRODUCT\}/\$\{P\}/g;
          $logicalIdentiferFormula  =~s/\$\{Field([\d]+)\}/\$\{F$1}/g;
          if ($deviceIdentifer && $iVendorId && $iProductId) {       
             if ($logicalIdentiferFormula) {
               $OriginalConfig->{"ID_DEFS"}->{$iVendorId}->{$iProductId}->{"LUN_VDID"}= $logicalIdentiferFormula;       
             }else{
               $OriginalConfig->{"ID_DEFS"}->{$iVendorId}->{$iProductId}->{"LUN_VDID"}= "";    
             }
          }       
    }
 
    $OriginalConfig->{"CAPABILITIES"}->{"REPLICATION"} = 0;
    $OriginalConfig->{"CAPABILITIES"}->{"PRIMARY_REMOTE_REP"} =0;
    if ("DiscoverReplications" eq $query->cookie('chDiscoverReplications')) {    
      $OriginalConfig->{"CAPABILITIES"}->{"REPLICATION"} = 1;    
      if ("DiscoverRemoteReplication" eq $query->cookie('chDiscoverRemoteReplication') ) {
        $OriginalConfig->{"CAPABILITIES"}->{"PRIMARY_REMOTE_REP"} = 1;      
      }     
    }

    $OriginalConfig->{"CAPABILITIES"}->{"TP"} = 0;
    $OriginalConfig->{"OBJECT_NAMES"}->{"THINPOOL"} = "";
    $OriginalConfig->{"CAPABILITIES"}->{"TP_LDEV_MAP"} = 0;
    $OriginalConfig->{"CAPABILITIES"}->{"TP_SOURCE_LDEV_MAP"} = 0; 
    $OriginalConfig->{"CAPABILITIES"}->{"TP_SOURCE_PDEV_MAP"} = 0; 
    if ("DiscoverThinPools" eq $query->cookie('chDiscoverThinPools')) {        
      $OriginalConfig->{"CAPABILITIES"}->{"TP"} = 1;
      my $iThinPoolTerm = $query->cookie('iThinPoolTerm');
      if ($iThinPoolTerm) {
          $OriginalConfig->{"OBJECT_NAMES"}->{"THINPOOL"} = $iThinPoolTerm;
      }

      if ("ThinPoolLogicalDiskMapping" eq $query->cookie('chThinPoolLogicalDiskMapping')) {
         if ( "DiscoverLogicalDevices" eq $query->cookie('chDiscoverLogicalDisk')) {
            $OriginalConfig->{"CAPABILITIES"}->{"TP_LDEV_MAP"} = 1;
         }         
      }
      if ("ThinPoolSourceDeviceMapping" eq $query->cookie('chThinPoolSourceDeviceMapping')) {   
          if ("LogicalThingPool" eq $query->cookie('chLogicalThingPool')) {   
            $OriginalConfig->{"CAPABILITIES"}->{"TP_SOURCE_LDEV_MAP"} = 1; 
          }
          if ("PhysicalThingPool" eq $query->cookie('chPhysicalThingPool')) {   
            $OriginalConfig->{"CAPABILITIES"}->{"TP_SOURCE_PDEV_MAP"} = 1; 
          }
      }
    }

    my $test =  $query->param("runTest");
    if ("yes" eq $test) {
         $OriginalConfig = updateTestConfigParam($OriginalConfig, $query);	
    }else{
       $OriginalConfig = getConfigParameters($OriginalConfig, $query);
    }
    return $OriginalConfig;
}

sub getConfigParameters{
    my ($OriginalConfig, $query) = @_;

    my $configParamSets = $query->cookie('iConfigureParams');	
    my @configParamArray = split(':::', $configParamSets);
    my $size = @configParamArray;
    my $configParmCount = 0;
    my $vendor = $query->cookie('iVendor');
    my $model = $query->cookie('iModel'); 
    my $mothod = $query->cookie('iFamily');
    foreach my $oneSet (@configParamArray) {	
        if (length($oneSet)>3) {
            my @params = split(',,,', $oneSet);
            my $displayName = @params[0];
            my $cliName = @params[1];
            my $defaultValue = @params[2];
            my $dataType = @params[3];
            my $mandatory = @params[4];
            my $testValue = @params[5];
            my $configParmEntry = join "_", $vendor, $model, $mothod, "field", $configParmCount;
            $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"display_name"} = $displayName;
            $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"cli_arg_name"} = $cliName;
            $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"default_value"}= $defaultValue;
            $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"datatype"}     = $dataType;
            $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"is_mandatory"} = $mandatory ; 
            if (defined  $testValue) {
                $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"test_value"} = $testValue;
            }else{
                $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"test_value"} = "";
            }
            $configParmCount = $configParmCount + 1;
        }
    }

  return $OriginalConfig;
}

sub updateTestConfigParam{
    my ($OriginalConfig, $query) = @_;
    my $configParamSet ="";
    my $configParmCount = 0;

    foreach my $key ($query->param) {
        my $configParmEntry = join "", "field", "_",  $configParmCount;
        my $oneConfigParam = "";
        if ($key =~ /^nameField_(.+)+/) {
             my $numberPart = $1;
             my $cliNameFieldKey = join "", "cliNameField", "_",  $numberPart;
             my $defaultValueFieldKey = join "", "defaultValueField", "_",  $numberPart;
             my $dataTypeSelectKey = join "", "dataTypeSelect", "_",  $numberPart;
             my $mandatorySelectKey = join "", "MandatorySelect", "_",  $numberPart;
             my $testValueFieldKey =  join "", "testValueField", "_",  $numberPart;

             my $nameField = $query->param($key);
             my $cliNameField = $query->param($cliNameFieldKey);
             my $defaultValueField = $query->param($defaultValueFieldKey);
             my $dataTypeSelectField = $query->param($dataTypeSelectKey);
             my $mandatorySelectField = $query->param($mandatorySelectKey);
             my $testValueField = $query->param($testValueFieldKey);			
        
            if ($nameField) {
                $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"display_name"} = $query->param($key);
                $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"cli_arg_name"} = $query->param($cliNameFieldKey);      
                $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"default_value"}= $query->param($defaultValueFieldKey);
                $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"datatype"}     = $query->param($dataTypeSelectKey);
                $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"is_mandatory"} = $query->param($mandatorySelectKey);   
                $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"test_value"} = "";
                if (defined  $testValueField) {
                    $OriginalConfig->{"DEVICE_CONFIG_META"}->{$configParmEntry}->{"test_value"} = $testValueField;
                }
                $oneConfigParam =  join ",,,", $nameField, $cliNameField, $defaultValueField, $dataTypeSelectField, $mandatorySelectField, $testValueField;   
                $configParamSet = join ":::", $configParamSet, $oneConfigParam; 
                $configParmCount = $configParmCount + 1;
            }
        } 
    }
    my $configParmCookie= $query->cookie(-name=>"iConfigureParams", -value=> $configParamSet);	
    print "Set-Cookie:$configParmCookie;\n"; 

    my $testCommandFile = $query->param("testCommandFile");
    my $testCommandFileCookie= $query->cookie(-name=>"testCommandFile", -value=> $testCommandFile);	
    print "Set-Cookie:$testCommandFileCookie;\n";

    my $testOption = $query->param("testOption"); 
    my $testOptionCookie= $query->cookie(-name=>"testOption", -value=> $testOption);	
    print "Set-Cookie:$testOptionCookie;\n";
    
    return $OriginalConfig;
}

sub jsonConfigToCookie{
    my $conf_json=shift;
    my $query = shift;
    my $iName = $conf_json->{"ARRAY_PLUGIN_NAME"};    
    my $iNameCookie= $query->cookie(-name=>"iName", -value=>"$iName");
    print "Set-Cookie:$iNameCookie;\n"; 

    my $iVendor = $conf_json->{"VENDOR"};
    my $iVendorCookie= $query->cookie(-name=>"iVendor", -value=>"$iVendor");	
    print "Set-Cookie:$iVendorCookie;\n"; 
 
    my $iVersion = $conf_json->{"ARRAY_PLUGIN_VERSION"};    
    my $iVersionCookie= $query->cookie(-name=>"iVersion", -value=>"$iVersion");
    print "Set-Cookie:$iVersionCookie;\n"; 

    my $iModel = $conf_json->{"MODEL"};    
    my $iModelCookie= $query->cookie(-name=>"iModel", -value=>"$iModel");
    print "Set-Cookie:$iModelCookie;\n"; 

    my $iFamily = $conf_json->{"ARRAY_PLUGIN_METHOD"};    
    my $iFamilyCookie= $query->cookie(-name=>"iFamily", -value=>"$iFamily");
    print "Set-Cookie:$iFamilyCookie;\n"; 

    my $cliCommand = $conf_json->{"DISCOVERY_INFO"}->{"ENCL_INFO"};    
    my $cliCommandCookie= $query->cookie(-name=>"cliCommand", -value=>"$cliCommand");
    print "Set-Cookie:$cliCommandCookie;\n"; 

    my $chDiscoveryPhysicalDiskCookie= $query->cookie(-name=>"chDiscoveryPhysicalDisk", -value=>"");
    my $iPhysicalDiskTermCookie = $query->cookie(-name=>"iPhysicalDiskTerm", -value=>""); 
    if ($conf_json->{"CAPABILITIES"}->{"PDEV"} == 1){
        $chDiscoveryPhysicalDiskCookie= $query->cookie(-name=>"chDiscoveryPhysicalDisk", -value=>"DiscoveryPhysicalDisk");    
    }
    my $iPhysicalDiskTerm = $conf_json->{"OBJECT_NAMES"}->{"PDEV"};
    if ( defined $iPhysicalDiskTerm){
        $iPhysicalDiskTermCookie = $query->cookie(-name=>"iPhysicalDiskTerm", -value=>"$iPhysicalDiskTerm"); 
    }
    print "Set-Cookie:$chDiscoveryPhysicalDiskCookie;\n";       
    print "Set-Cookie:$iPhysicalDiskTermCookie;\n";  
    
    # Discover Physical Groups not used in backend
    # iPhysicalGroupTerm
    
    my $chDiscoverArrayPortCookie= $query->cookie(-name=>"chDiscoverArrayPort", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"ARRAY_PORT"} == 1){
        $chDiscoverArrayPortCookie= $query->cookie(-name=>"chDiscoverArrayPort", -value=>"DiscoverArrayPort");    
    }
    print "Set-Cookie:$chDiscoverArrayPortCookie;\n";       

    my $chDiscoverAdapterCookie= $query->cookie(-name=>"chDiscoverAdapter", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"ADAPTER"} == 1){
        $chDiscoverAdapterCookie= $query->cookie(-name=>"chDiscoverAdapter", -value=>"DiscoverAdapter");    
    }
    print "Set-Cookie:$chDiscoverAdapterCookie;\n";       

    my $chDiscoverRaidGroupCookie= $query->cookie(-name=>"chDiscoverRaidGroup", -value=>"");
    if ( $conf_json->{"CAPABILITIES"}->{"RG"} == 1) {
       $chDiscoverRaidGroupCookie= $query->cookie(-name=>"chDiscoverRaidGroup", -value=>"DiscoverRaidGroup");	
    }
    print "Set-Cookie:$chDiscoverRaidGroupCookie;\n"; 
    
    my $iRaidGroupTermCookie= $query->cookie(-name=>"iRaidGroupTerm", -value=>"");
    my $iRaidGroupTerm = $conf_json->{"OBJECT_NAMES"}->{"RAIDGROUP"};
    if ( defined $iRaidGroupTerm) {
       $iRaidGroupTermCookie= $query->cookie(-name=>"iRaidGroupTerm", -value=>"$iRaidGroupTerm");
    }
    print "Set-Cookie:$iRaidGroupTermCookie;\n"; 

    my $chRaidGroupPhysicalMappingCookie= $query->cookie(-name=>"chRaidGroupPhysicalMapping", -value=>"");
    if ( $conf_json->{"CAPABILITIES"}->{"RG_PDEV_MAP"} == 1) {
       $chRaidGroupPhysicalMappingCookie= $query->cookie(-name=>"chRaidGroupPhysicalMapping", -value=>"RaidGroupPhysicalMapping");
    }
    print "Set-Cookie:$chRaidGroupPhysicalMappingCookie;\n"; 
    
    my $chRaidGroupLogicalDiskMappingCookie= $query->cookie(-name=>"chRaidGroupLogicalDiskMapping", -value=>"");
    if ( $conf_json->{"CAPABILITIES"}->{"RG_PDEV_MAP"} == 1) {
       $chRaidGroupLogicalDiskMappingCookie= $query->cookie(-name=>"chRaidGroupLogicalDiskMapping", -value=>"RaidGroupLogicalDiskMapping");
    }
    print "Set-Cookie:$chRaidGroupLogicalDiskMappingCookie;\n"; 

    my $chDiscoverLogicalDiskCookie= $query->cookie(-name=>"chDiscoverLogicalDisk", -value=>"");
    my $iLogicalDiskTermCookie = $query->cookie(-name=>"iLogicalDiskTerm", -value=>""); 
    if ($conf_json->{"CAPABILITIES"}->{"LDEV"} == 1){
        $chDiscoverLogicalDiskCookie= $query->cookie(-name=>"chDiscoverLogicalDisk", -value=>"DiscoverLogicalDevices");   
    }
    my $iLogicalDiskTerm = $conf_json->{"OBJECT_NAMES"}->{"LDEV"};
    if ( defined $iLogicalDiskTerm){
        $iLogicalDiskTermCookie = $query->cookie(-name=>"iLogicalDiskTerm", -value=>"$iLogicalDiskTerm"); 
    }
    print "Set-Cookie:$chDiscoverLogicalDiskCookie;\n";       
    print "Set-Cookie:$iLogicalDiskTermCookie;\n";  
     
    my $chDiscoverLogPhyMappingCookie= $query->cookie(-name=>"chDiscoverLogPhyMapping", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"LDEV_PDEV_MAP"} == 1){
        $chDiscoverLogPhyMappingCookie= $query->cookie(-name=>"chDiscoverLogPhyMapping", -value=>"DiscoverLogPhyMapping");    
    }
    print "Set-Cookie:$chDiscoverLogPhyMappingCookie;\n";       
    my $chDiscoverLogHostMappingCookie= $query->cookie(-name=>"chDiscoverLogHostMapping", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"HOST_LUN_MAPPING"} == 1){
        $chDiscoverLogHostMappingCookie= $query->cookie(-name=>"chDiscoverLogHostMapping", -value=>"DiscoverLogHostMapping");    
    }
    print "Set-Cookie:$chDiscoverLogHostMappingCookie;\n";       

    my $iTermMetaLogicalDisk = $conf_json->{"OBJECT_NAMES"}->{"META_LDEV"};
    my $chMetaLogicalDiskCookie = $query->cookie(-name=>"chMetaLogicalDisk", -value=>"");
    my $iTermMetaLogicalDiskCookie= $query->cookie(-name=>"iTermMetaLogicalDisk", -value=>"");    
    if ($iTermMetaLogicalDisk){
       $chMetaLogicalDiskCookie = $query->cookie(-name=>"chMetaLogicalDisk", -value=>"MetaLogicalDisk");
       $iTermMetaLogicalDiskCookie= $query->cookie(-name=>"iTermMetaLogicalDisk", -value=>"$iTermMetaLogicalDisk"); 
    }
    print "Set-Cookie:$chMetaLogicalDiskCookie;\n";       
    print "Set-Cookie:$iTermMetaLogicalDiskCookie;\n";  

    my $chDiscoverReplicationsCookie= $query->cookie(-name=>"chDiscoverReplications", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"REPLICATION"} == 1){
        $chDiscoverReplicationsCookie= $query->cookie(-name=>"chDiscoverReplications", -value=>"DiscoverReplications");    
    }
    print "Set-Cookie:$chDiscoverReplicationsCookie;\n";
   
    my $chDiscoverRemoteReplicationCookie= $query->cookie(-name=>"chDiscoverRemoteReplication", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"PRIMARY_REMOTE_REP"} == 1){
        $chDiscoverRemoteReplicationCookie= $query->cookie(-name=>"chDiscoverRemoteReplication", -value=>"DiscoverRemoteReplication");    
    }
    print "Set-Cookie:$chDiscoverRemoteReplicationCookie;\n";

    # thin pool
    my $chDiscoverThinPoolsCookie= $query->cookie(-name=>"chDiscoverThinPools", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"TP"} == 1){
        $chDiscoverThinPoolsCookie= $query->cookie(-name=>"chDiscoverThinPools", -value=>"DiscoverThinPools");    
    }
    print "Set-Cookie:$chDiscoverThinPoolsCookie;\n";
    
    my $iThinPoolTermCookie = $query->cookie(-name=>"iThinPoolTerm", -value=>"");
    my $iThinPoolTerm = $conf_json->{"OBJECT_NAMES"}->{"THINPOOL"};
    if ( defined $iThinPoolTerm){
        $iThinPoolTermCookie = $query->cookie(-name=>"iThinPoolTerm", -value=>"$iThinPoolTerm"); 
    }
    print "Set-Cookie:$iThinPoolTermCookie;\n";       

    my $chThinPoolLogicalDiskMappingCookie= $query->cookie(-name=>"chThinPoolLogicalDiskMapping", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"TP_LDEV_MAP"} == 1){
        $chThinPoolLogicalDiskMappingCookie= $query->cookie(-name=>"chThinPoolLogicalDiskMapping", -value=>"ThinPoolLogicalDiskMapping");    
    }
    print "Set-Cookie:$chThinPoolLogicalDiskMappingCookie;\n";
    
    my $chLogicalThingPoolCookie= $query->cookie(-name=>"chLogicalThingPool", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"TP_SOURCE_LDEV_MAP"} == 1){
        $chLogicalThingPoolCookie= $query->cookie(-name=>"chLogicalThingPool", -value=>"LogicalThingPool");    
    }
    print "Set-Cookie:$chLogicalThingPoolCookie;\n";

    my $chPhysicalThingPoolCookie= $query->cookie(-name=>"chPhysicalThingPool", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"TP_SOURCE_PDEV_MAP"} == 1){
        $chPhysicalThingPoolCookie= $query->cookie(-name=>"chPhysicalThingPool", -value=>"PhysicalThingPool");    
    }
    print "Set-Cookie:$chPhysicalThingPoolCookie;\n";

    my $chThinPoolSourceDeviceMappingCookie= $query->cookie(-name=>"chThinPoolSourceDeviceMapping", -value=>"");
    if ($conf_json->{"CAPABILITIES"}->{"TP_SOURCE_PDEV_MAP"} == 1 || $conf_json->{"CAPABILITIES"}->{"TP_SOURCE_LDEV_MAP"} == 1 ){
        $chThinPoolSourceDeviceMappingCookie= $query->cookie(-name=>"chThinPoolSourceDeviceMapping", -value=>"ThinPoolSourceDeviceMapping");    
    }
    print "Set-Cookie:$chThinPoolSourceDeviceMappingCookie;\n";


    #end of thin pool
    
    my $commandScript = $conf_json->{"DISCOVERY_INFO"}->{"COMMAND_LOCATION"};    
    my $commandScriptCookie= $query->cookie(-name=>"commandScript", -value=>"$commandScript");
    print "Set-Cookie:$commandScriptCookie;\n"; 

    my $iDiscoveryFileName = $conf_json->{"DISCOVERY_INFO"}->{"DISCOVERY_CMD"};    
    my $iDiscoveryFileNameCookie= $query->cookie(-name=>"iDiscoveryFileName", -value=>"$iDiscoveryFileName");
    print "Set-Cookie:$iDiscoveryFileNameCookie;\n"; 

    my $configParamSet = jsonConfigToCliParameterCookie($conf_json, $query);
    jsonConfigToDeviceIdentifierCookie($conf_json, $query);

    VRTS::Logger::log('debug', "finished jsonConfigToCookie");
    return $configParamSet;
}
sub jsonConfigToCliParameterCookie{
    my $conf_json=shift;
    my $query = shift;

    my  $configParamSet = "";
    foreach my $key (keys %{$conf_json->{DEVICE_CONFIG_META}})
    {
       my $oneConfigParam = "";
       if ($conf_json->{DEVICE_CONFIG_META}->{$key}->{cli_arg_name}) {  
          my $nameField = $conf_json->{DEVICE_CONFIG_META}->{$key}->{display_name};
          my $cliNameField = $conf_json->{DEVICE_CONFIG_META}->{$key}->{cli_arg_name};
          my $defaultValueField = $conf_json->{DEVICE_CONFIG_META}->{$key}->{default_value};
          my $dataTypeSelectField = $conf_json->{DEVICE_CONFIG_META}->{$key}->{datatype};
          my $mandatorySelectField =$conf_json->{DEVICE_CONFIG_META}->{$key}->{is_mandatory};
          if ($mandatorySelectField ==1) {
            $mandatorySelectField = "Yes";
          }else{
            $mandatorySelectField = "No";
          }
          $oneConfigParam =  join ",,,", $nameField, $cliNameField, $defaultValueField, $dataTypeSelectField, $mandatorySelectField, "";          
          $configParamSet = join ":::", $configParamSet, $oneConfigParam; 
       }    
    }
    my $configParmCookie= $query->cookie(-name=>"iConfigureParams", -value=> $configParamSet);
    print "Set-Cookie:$configParmCookie;\n";  
    return $configParamSet;
}

sub jsonConfigToDeviceIdentifierCookie{
    my $conf_json=shift;
    my $query = shift;

    my @vendorArray  = keys %{$conf_json->{ID_DEFS}};
    my $firstVendor =  $vendorArray[0];

    my $iSSIVendor= $query->cookie(-name=>"iVendorId", -value=> $firstVendor);
    print "Set-Cookie:$iSSIVendor;\n";  

    my @productArray = keys %{$conf_json->{ID_DEFS}->{$firstVendor}};
    my $firstProduct = $productArray[0];

    my $iSSIVendor= $query->cookie(-name=>"iProductId", -value=> $firstProduct);
    print "Set-Cookie:$iSSIVendor;\n";  

    my $encIdentiferFormula = $conf_json->{ID_DEFS}->{$firstVendor}->{$firstProduct}->{ENCL_VDID};                                          
    $encIdentiferFormula  =~s/\$\{V\}/\$\{VENDOR\}/g;
    $encIdentiferFormula  =~s/\$\{P\}/\$\{PRODUCT\}/g;
    $encIdentiferFormula  =~s/\$\{F([\d]+)\}/\$\{Field$1\}/g;
    my $encIdentiferFormulaCookie= $query->cookie(-name=>"EncIdentiferFormula", -value=> $encIdentiferFormula);
    print "Set-Cookie:$encIdentiferFormulaCookie;\n";  

    my $logicalIdentiferFormula = $conf_json->{ID_DEFS}->{$firstVendor}->{$firstProduct}->{LUN_VDID};
    $logicalIdentiferFormula  =~s/\$\{V\}/\$\{VENDOR\}/g;
    $logicalIdentiferFormula  =~s/\$\{P\}/\$\{PRODUCT\}/g;
    $logicalIdentiferFormula  =~s/\$\{F([\d]+)\}/\$\{Field$1\}/g;
    my $logicalIdentiferFormulaCookie= $query->cookie(-name=>"LogicalIdentiferFormula", -value=> $logicalIdentiferFormula);
    print "Set-Cookie:$logicalIdentiferFormulaCookie;\n";  

    my $feild_defs = $conf_json->{ID_DEFS}->{$firstVendor}->{$firstProduct}->{FIELDS};  
    my $buf = JSON::objToJson($feild_defs);
    my $deviceIdentiferCookie= $query->cookie(-name=>"deviceIdentifer", -value=> $buf);
    print "Set-Cookie:$deviceIdentiferCookie;\n";  
    VRTS::Logger::log('debug', "Device Identifier from existing file: $deviceIdentiferCookie");

}

sub getAddOnConfig{
    my $query = shift;
    my $appdir = shift;
    my $installdir = shift;
    my $baseInputSfaFileName = $query->param("iSfaFile");

    VRTS::Logger::log('debug', "file name: $baseInputSfaFileName appdir:$appdir installdir:$installdir");
    my $sisdkAddOnTempDir = File::Temp::tempnam ($appdir, "sisdkAddOn");
    VRTS::Logger::log('debug', "<getAddOnConfig temp directory>". $sisdkAddOnTempDir);

    eval ( File::Path::mkpath($sisdkAddOnTempDir, 0, 0755) );
    if($@)
    {
        my $msg = "Internal Error";
        VRTS::Logger::log('debug',"Failed to create directory $sisdkAddOnTempDir:$@");
        return 1, $msg;
    }  
    VRTS::Logger::log('debug',"input file name: $baseInputSfaFileName temp directory $sisdkAddOnTempDir:$@");
    my ($ret, $uploadedFileName) = &handleFileUpload($baseInputSfaFileName,$sisdkAddOnTempDir,$installdir);        
    my $add_on_config ="";
    if ($ret) {
       return ($ret, $uploadedFileName);
    }  
    eval {
       $add_on_config = VRTS::SISdk::Fabricate::extract_plugin($uploadedFileName, $sisdkAddOnTempDir);     
    };

    if ($@)
    {
        my $msg = "Failed to extract plugin $sisdkAddOnTempDir:$@";
        VRTS::Logger::log('error', $msg);
        File::Path::rmtree($sisdkAddOnTempDir);
        return 1, $msg;
    }
    my $sisdkAddOnTempDirCookie= $query->cookie(-name=>"sisdkAddOnTempDir", -value=>"$sisdkAddOnTempDir");	
    print "Set-Cookie:$sisdkAddOnTempDirCookie;\n"; 

    return (0, $add_on_config);
}
